#
# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

'''
Senlin exception subclasses.
'''

import sys

from oslo_log import log as logging
import six

from senlin.common.i18n import _

_FATAL_EXCEPTION_FORMAT_ERRORS = False
LOG = logging.getLogger(__name__)


class SenlinException(Exception):
    """Base Senlin Exception.

    To correctly use this class, inherit from it and define a 'msg_fmt'
    property. That msg_fmt will get printed with the keyword arguments
    provided to the constructor.
    """
    message = _("An unknown exception occurred.")

    def __init__(self, **kwargs):
        self.kwargs = kwargs

        try:
            self.message = self.msg_fmt % kwargs
            # if last char is '.', wipe out redundant '.'
            if self.message[-1] == '.':
                self.message = self.message.rstrip('.') + '.'
        except KeyError:
            # exc_info = sys.exc_info()
            # if kwargs doesn't match a variable in the message
            # log the issue and the kwargs
            LOG.exception('Exception in string format operation')
            for name, value in kwargs.items():
                LOG.error("%s: %s" % (name, value))  # noqa

            if _FATAL_EXCEPTION_FORMAT_ERRORS:
                raise
                # raise exc_info[0], exc_info[1], exc_info[2]

    def __str__(self):
        return six.text_type(self.message)

    def __unicode__(self):
        return six.text_type(self.message)

    def __deepcopy__(self, memo):
        return self.__class__(**self.kwargs)


class SIGHUPInterrupt(SenlinException):
    msg_fmt = _("System SIGHUP signal received.")


class NotAuthenticated(SenlinException):
    msg_fmt = _("You are not authenticated.")


class Forbidden(SenlinException):
    msg_fmt = _("You are not authorized to complete this operation.")


class BadRequest(SenlinException):
    msg_fmt = _("%(msg)s.")


class InvalidAPIVersionString(SenlinException):
    msg_fmt = _("API Version String '%(version)s' is of invalid format. It "
                "must be of format 'major.minor'.")


class MethodVersionNotFound(SenlinException):
    msg_fmt = _("API version '%(version)s' is not supported on this method.")


class InvalidGlobalAPIVersion(SenlinException):
    msg_fmt = _("Version '%(req_ver)s' is not supported by the API. Minimum "
                "is '%(min_ver)s' and maximum is '%(max_ver)s'.")


class MultipleChoices(SenlinException):
    msg_fmt = _("Multiple results found matching the query criteria "
                "'%(arg)s'. Please be more specific.")


class ResourceNotFound(SenlinException):
    """Generic exception for resource not found.

    The resource type here can be 'cluster', 'node', 'profile',
    'policy', 'receiver', 'webhook', 'profile_type', 'policy_type',
    'action', 'event' and so on.
    """
    msg_fmt = _("The %(type)s '%(id)s' could not be found.")

    @staticmethod
    def enhance_msg(enhance, ex):
        enhance_msg = ex.message[:4] + enhance + ' ' + ex.message[4:]
        return enhance_msg


class ResourceInUse(SenlinException):
    """Generic exception for resource in use.

    The resource type here can be 'cluster', 'node', 'profile',
    'policy', 'receiver', 'webhook', 'profile_type', 'policy_type',
    'action', 'event' and so on.
    """
    msg_fmt = _("The %(type)s '%(id)s' cannot be deleted: %(reason)s.")


class ProfileNotSpecified(SenlinException):
    msg_fmt = _("Profile not specified.")


class ProfileOperationFailed(SenlinException):
    msg_fmt = _("%(message)s")


class ProfileOperationTimeout(SenlinException):
    msg_fmt = _("%(message)s")


class PolicyNotSpecified(SenlinException):
    msg_fmt = _("Policy not specified.")


class PolicyBindingNotFound(SenlinException):
    msg_fmt = _("The policy '%(policy)s' is not found attached to the "
                "specified cluster '%(identity)s'.")


class PolicyTypeConflict(SenlinException):
    msg_fmt = _("The policy with type '%(policy_type)s' already exists.")


class InvalidSpec(SenlinException):
    msg_fmt = _("%(message)s")


class FeatureNotSupported(SenlinException):
    msg_fmt = _("%(feature)s is not supported.")


class Error(SenlinException):
    msg_fmt = "%(message)s"

    def __init__(self, msg):
        super(Error, self).__init__(message=msg)


class InvalidContentType(SenlinException):
    msg_fmt = _("Invalid content type %(content_type)s")


class RequestLimitExceeded(SenlinException):
    msg_fmt = _('Request limit exceeded: %(message)s')


class ActionInProgress(SenlinException):
    msg_fmt = _("The %(type)s '%(id)s' is in status %(status)s.")


class NodeNotOrphan(SenlinException):
    msg_fmt = _("%(message)s")


class InternalError(SenlinException):
    """A base class for internal exceptions in senlin.

    The internal exception classes which inherit from :class:`SenlinException`
    class should be translated to a user facing exception type if need to be
    made user visible.
    """
    msg_fmt = _("%(message)s")
    message = _('Internal error happened')

    def __init__(self, **kwargs):
        self.code = kwargs.pop('code', 500)
        self.message = kwargs.pop('message', self.message)
        super(InternalError, self).__init__(
            code=self.code, message=self.message, **kwargs)


class EResourceBusy(InternalError):
    # Internal exception, not to be exposed to end user.
    msg_fmt = _("The %(type)s '%(id)s' is busy now.")


class TrustNotFound(InternalError):
    # Internal exception, not to be exposed to end user.
    msg_fmt = _("The trust for trustor '%(trustor)s' could not be found.")


class EResourceCreation(InternalError):
    # Used when creating resources in other services
    def __init__(self, **kwargs):
        self.resource_id = kwargs.pop('resource_id', None)
        super(EResourceCreation, self).__init__(
            resource_id=self.resource_id, **kwargs)
    msg_fmt = _("Failed in creating %(type)s: %(message)s.")


class EResourceUpdate(InternalError):
    # Used when updating resources from other services
    msg_fmt = _("Failed in updating %(type)s '%(id)s': %(message)s.")


class EResourceDeletion(InternalError):
    # Used when deleting resources from other services
    msg_fmt = _("Failed in deleting %(type)s '%(id)s': %(message)s.")


class EServerNotFound(InternalError):
    # Used when deleting resources from other services
    msg_fmt = _("Failed in found %(type)s '%(id)s': %(message)s.")


class EResourceOperation(InternalError):
    """Generic exception for resource fail operation.

    The op here can be 'recovering','rebuilding', 'checking' and
    so on. And the op 'creating', 'updating' and 'deleting' we can
    use separately class `EResourceCreation`,`EResourceUpdate` and
    `EResourceDeletion`.
    The type here is resource's driver type.It can be 'server',
    'stack', 'container' and so on.
    The id is resource's id.
    The message here can be message from class 'ResourceNotFound',
    'ResourceInUse' and so on, or developer can specified message.
    """
    # Used when operating resources from other services
    msg_fmt = _("Failed in %(op)s %(type)s '%(id)s': %(message)s.")


class ESchema(InternalError):
    msg_fmt = _("%(message)s")


class InvalidPlugin(InternalError):
    msg_fmt = _("%(message)s")


class PolicyNotAttached(InternalError):
    msg_fmt = _("The policy '%(policy)s' is not attached to the specified "
                "cluster '%(cluster)s'.")


class HTTPExceptionDisguise(Exception):
    """Disguises HTTP exceptions.

    The purpose is to let them be handled by the webob fault application
    in the wsgi pipeline.
    """

    def __init__(self, exception):
        self.exc = exception
        self.tb = sys.exc_info()[2]



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy

from oslo_serialization import jsonutils
from senlin.objects.requests import profiles
from senlin.tests.unit.common import base as test_base


class TestProfileCreate(test_base.SenlinTestCase):

    spec = {
        'type': 'os.nova.server',
        'version': '1.0',
        'properties': {
            'name': 'FAKE_SERVER_NAME',
            'flavor': 'FAKE_FLAVOR',
            'image': 'FAKE_IMAGE',
            'key_name': 'FAKE_KEYNAME',
            'networks': [{'network': 'FAKE_NET'}],
            'user_data': 'FAKE_USER_DATA'
        }
    }

    def test_profile_create_body(self):
        spec = copy.deepcopy(self.spec)
        sot = profiles.ProfileCreateRequestBody(name='foo', spec=spec,
                                                metadata={'x': 'y'})
        self.assertEqual('foo', sot.name)
        self.assertEqual({'x': 'y'}, sot.metadata)
        self.assertEqual(u'os.nova.server', sot.spec['type'])
        self.assertEqual(u'1.0', sot.spec['version'])

    def test_profile_create_request(self):
        spec = copy.deepcopy(self.spec)
        body = profiles.ProfileCreateRequestBody(name='foo', spec=spec,
                                                 metadata={'x': 'y'})
        sot = profiles.ProfileCreateRequest(profile=body)
        self.assertIsInstance(sot.profile, profiles.ProfileCreateRequestBody)

    def test_request_body_to_primitive(self):
        spec = copy.deepcopy(self.spec)
        sot = profiles.ProfileCreateRequestBody(name='test-profile',
                                                spec=spec,
                                                metadata={'x': 'y'})
        self.assertEqual('test-profile', sot.name)
        self.assertEqual({'x': 'y'}, sot.metadata)

        res = sot.obj_to_primitive()
        # request body
        self.assertEqual('ProfileCreateRequestBody', res['senlin_object.name'])
        self.assertEqual('1.0', res['senlin_object.version'])
        self.assertEqual('senlin', res['senlin_object.namespace'])
        self.assertIn('name', res['senlin_object.changes'])
        self.assertIn('spec', res['senlin_object.changes'])
        self.assertIn('metadata', res['senlin_object.changes'])
        # spec
        data = res['senlin_object.data']
        self.assertEqual(u'test-profile', data['name'])
        self.assertEqual(u'{"x": "y"}', data['metadata'])
        # spec data
        spec_data = jsonutils.loads(data['spec'])
        self.assertEqual(u'os.nova.server', spec_data['type'])
        self.assertEqual(u'1.0', spec_data['version'])

    def test_request_to_primitive(self):
        spec = copy.deepcopy(self.spec)
        body = profiles.ProfileCreateRequestBody(name='test-profile',
                                                 spec=spec,
                                                 metadata={'x': 'y'})
        sot = profiles.ProfileCreateRequest(profile=body)
        self.assertIsInstance(sot.profile, profiles.ProfileCreateRequestBody)
        self.assertEqual('test-profile', sot.profile.name)
        self.assertEqual({'x': 'y'}, sot.profile.metadata)

        # request
        res = sot.obj_to_primitive()
        self.assertIn('profile', res['senlin_object.changes'])
        self.assertEqual('ProfileCreateRequest', res['senlin_object.name'])
        self.assertEqual('senlin', res['senlin_object.namespace'])
        self.assertEqual('1.0', res['senlin_object.version'])

        # request body
        data = res['senlin_object.data']['profile']
        self.assertEqual('ProfileCreateRequestBody',
                         data['senlin_object.name'])
        self.assertEqual('senlin', data['senlin_object.namespace'])
        self.assertEqual('1.0', data['senlin_object.version'])
        self.assertIn('name', data['senlin_object.changes'])
        self.assertIn('spec', data['senlin_object.changes'])
        self.assertIn('metadata', data['senlin_object.changes'])

        # spec
        pd = data['senlin_object.data']
        self.assertEqual(u'test-profile', pd['name'])

        spec_data = jsonutils.loads(pd['spec'])
        self.assertEqual(u'os.nova.server', spec_data['type'])
        self.assertEqual(u'1.0', spec_data['version'])


class TestProfileList(test_base.SenlinTestCase):

    def test_profile_list_request_body_full(self):
        params = {
            'name': ['p1'],
            'type': ['os.nova.server-1.0'],
            'limit': 2,
            'marker': 'd8d7dd1e-afd8-4921-83b2-c4ce73b1cb22',
            'sort': 'name:asc',
            'project_safe': False
        }
        sot = profiles.ProfileListRequest(**params)
        self.assertEqual(['p1'], sot.name)
        self.assertEqual(['os.nova.server-1.0'], sot.type)
        self.assertEqual(2, sot.limit)
        self.assertEqual('d8d7dd1e-afd8-4921-83b2-c4ce73b1cb22',
                         sot.marker)
        self.assertEqual('name:asc', sot.sort)
        self.assertFalse(sot.project_safe)


class TestProfileGet(test_base.SenlinTestCase):

    def test_profile_get(self):
        sot = profiles.ProfileGetRequest(identity='FAKE_ID')
        self.assertEqual('FAKE_ID', sot.identity)


class TestProfileUpdate(test_base.SenlinTestCase):

    def test_profile_update_body(self):
        data = {'name': 'foo', 'metadata': {'aaa': 'bbb'}}
        sot = profiles.ProfileUpdateRequestBody(**data)
        self.assertEqual('foo', sot.name)
        self.assertEqual({'aaa': 'bbb'}, sot.metadata)

    def test_profile_update(self):
        data = {'name': 'foo', 'metadata': {'aaa': 'bbb'}}
        body = profiles.ProfileUpdateRequestBody(**data)

        request = {
            'identity': 'pid',
            'profile': body
        }
        sot = profiles.ProfileUpdateRequest(**request)
        self.assertEqual('pid', sot.identity)
        self.assertIsInstance(sot.profile, profiles.ProfileUpdateRequestBody)

    def test_profile_data_to_primitive(self):
        data = {'name': 'foo', 'metadata': {'aaa': 'bbb'}}
        sot = profiles.ProfileUpdateRequestBody(**data)
        res = sot.obj_to_primitive()

        self.assertIn('name', res['senlin_object.changes'])
        self.assertIn('metadata', res['senlin_object.changes'])
        self.assertEqual('foo', res['senlin_object.data']['name'])
        self.assertEqual('{"aaa": "bbb"}',
                         res['senlin_object.data']['metadata'])
        self.assertEqual('ProfileUpdateRequestBody', res['senlin_object.name'])
        self.assertEqual('senlin', res['senlin_object.namespace'])
        self.assertEqual('1.0', res['senlin_object.version'])

    def test_request_to_primitive(self):
        data = {'name': 'foo', 'metadata': {'aaa': 'bbb'}}
        body = profiles.ProfileUpdateRequestBody(**data)

        request = {
            'identity': 'pid',
            'profile': body
        }
        sot = profiles.ProfileUpdateRequest(**request)
        res = sot.obj_to_primitive()

        self.assertIn('identity', res['senlin_object.changes'])
        self.assertEqual(u'pid', res['senlin_object.data']['identity'])
        self.assertEqual('ProfileUpdateRequest', res['senlin_object.name'])
        self.assertEqual('senlin', res['senlin_object.namespace'])
        self.assertEqual('1.0', res['senlin_object.version'])


class TestProfileValidate(test_base.SenlinTestCase):
    spec = {
        'type': 'os.nova.server',
        'version': '1.0',
        'properties': {
            'name': 'FAKE_SERVER_NAME',
            'flavor': 'FAKE_FLAVOR',
            'image': 'FAKE_IMAGE',
            'key_name': 'FAKE_KEYNAME',
            'networks': [{'network': 'FAKE_NET'}],
            'user_data': 'FAKE_USER_DATA'
        }
    }

    def test_validate_request_body(self):
        spec = copy.deepcopy(self.spec)
        body = profiles.ProfileValidateRequestBody(spec=spec)

        self.assertEqual('os.nova.server', body.spec['type'])
        self.assertEqual('1.0', body.spec['version'])

    def test_validate_request(self):
        spec = copy.deepcopy(self.spec)
        body = profiles.ProfileValidateRequestBody(spec=spec)

        sot = profiles.ProfileValidateRequest(profile=body)
        self.assertIsInstance(sot.profile, profiles.ProfileValidateRequestBody)

    def test_request_body_to_primitive(self):
        spec = copy.deepcopy(self.spec)
        body = profiles.ProfileValidateRequestBody(spec=spec)
        res = body.obj_to_primitive()

        self.assertIn('spec', res['senlin_object.changes'])
        self.assertEqual(
            'ProfileValidateRequestBody', res['senlin_object.name'])
        self.assertEqual('senlin', res['senlin_object.namespace'])
        self.assertEqual('1.0', res['senlin_object.version'])

        data = jsonutils.loads(res['senlin_object.data']['spec'])
        self.assertEqual(u'os.nova.server', data['type'])
        self.assertEqual(u'1.0', data['version'])

    def test_request_to_primitive(self):
        spec = copy.deepcopy(self.spec)
        body = profiles.ProfileValidateRequestBody(spec=spec)

        sot = profiles.ProfileValidateRequest(profile=body)
        res = sot.obj_to_primitive()

        self.assertIn('profile', res['senlin_object.changes'])
        self.assertEqual('ProfileValidateRequest', res['senlin_object.name'])
        self.assertEqual('senlin', res['senlin_object.namespace'])
        self.assertEqual('1.0', res['senlin_object.version'])

        profile_body = res['senlin_object.data']['profile']
        self.assertIn('spec', profile_body['senlin_object.changes'])

        self.assertEqual(
            'ProfileValidateRequestBody', profile_body['senlin_object.name'])
        self.assertEqual('senlin', profile_body['senlin_object.namespace'])
        self.assertEqual('1.0', profile_body['senlin_object.version'])

        data = jsonutils.loads(profile_body['senlin_object.data']['spec'])
        self.assertEqual(u'os.nova.server', data['type'])
        self.assertEqual(u'1.0', data['version'])


class TestProfileDelete(test_base.SenlinTestCase):

    def test_profile_delete(self):
        sot = profiles.ProfileDeleteRequest(identity='FAKE_ID')

        self.assertEqual('FAKE_ID', sot.identity)



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from six.moves.urllib import parse as urllib
import time

from oslo_serialization import jsonutils
from oslo_utils import timeutils
from tempest import config
from tempest.lib.common import rest_client
from tempest.lib import exceptions

CONF = config.CONF


class ClusteringAPIClient(rest_client.RestClient):
    version = 'v1'
    api_microversion = 'latest'

    def get_headers(self, accept_type=None, send_type=None):
        headers = super(ClusteringAPIClient, self).get_headers(
            accept_type=accept_type, send_type=send_type)
        headers['openstack-api-version'] = ('clustering %s' %
                                            self.api_microversion)
        return headers

    def get_resp(self, resp, body):
        # Parse status code and location
        res = {
            'status': int(resp.pop('status')),
            'location': resp.pop('location', None)
        }
        # Parse other keys included in resp
        res.update(resp)

        # Parse body
        if body and str(body) != 'null':
            res['body'] = self._parse_resp(body)
        else:
            res['body'] = None

        return res

    def get_obj(self, obj_type, obj_id, params=None):
        uri = '{0}/{1}/{2}'.format(self.version, obj_type, obj_id)
        if params:
            uri += '?{0}'.format(urllib.urlencode(params))
        resp, body = self.get(uri)

        return self.get_resp(resp, body)

    def create_obj(self, obj_type, attrs):
        uri = '{0}/{1}'.format(self.version, obj_type)
        resp, body = self.post(uri, body=jsonutils.dumps(attrs))

        return self.get_resp(resp, body)

    def list_objs(self, obj_type, params=None):
        uri = '{0}/{1}'.format(self.version, obj_type)
        if params:
            uri += '?{0}'.format(urllib.urlencode(params))
        resp, body = self.get(uri)

        return self.get_resp(resp, body)

    def update_obj(self, obj_type, obj_id, attrs):
        uri = '{0}/{1}/{2}'.format(self.version, obj_type, obj_id)
        resp, body = self.patch(uri, body=jsonutils.dumps(attrs))

        return self.get_resp(resp, body)

    def delete_obj(self, obj_type, obj_id):
        uri = '{0}/{1}/{2}'.format(self.version, obj_type, obj_id)
        resp, body = self.delete(uri)

        return self.get_resp(resp, body)

    def validate_obj(self, obj_type, attrs):
        uri = '{0}/{1}/validate'.format(self.version, obj_type)
        headers = {'openstack-api-version': 'clustering 1.2'}
        resp, body = self.post(uri, body=jsonutils.dumps(attrs),
                               headers=headers)

        return self.get_resp(resp, body)

    def trigger_webhook(self, webhook_url, params=None):
        if params is not None:
            params = jsonutils.dumps(params)
        resp, body = self.raw_request(webhook_url, 'POST', body=params)
        return self.get_resp(resp, body)

    def trigger_action(self, obj_type, obj_id, params=None):
        uri = '{0}/{1}/{2}/actions'.format(self.version, obj_type, obj_id)
        if params is not None:
            params = jsonutils.dumps(params)
        resp, body = self.post(uri, body=params)

        return self.get_resp(resp, body)

    def trigger_operation(self, obj_type, obj_id, params):
        uri = '{0}/{1}/{2}/ops'.format(self.version, obj_type, obj_id)
        data = jsonutils.dumps(params)
        resp, body = self.post(uri, body=data)
        return self.get_resp(resp, body)

    def cluster_replace_nodes(self, obj_type, obj_id, params=None):
        uri = '{0}/{1}/{2}/actions'.format(self.version, obj_type, obj_id)
        if params is not None:
            params = jsonutils.dumps(params)
        headers = self.get_headers()
        resp, body = self.post(uri, body=params, headers=headers)

        return self.get_resp(resp, body)

    def list_cluster_policies(self, cluster_id, params=None):
        uri = '{0}/clusters/{1}/policies'.format(self.version, cluster_id)
        if params:
            uri += '?{0}'.format(urllib.urlencode(params))

        resp, body = self.get(uri)

        return self.get_resp(resp, body)

    def get_cluster_policy(self, cluster_id, policy_id):
        uri = '{0}/clusters/{1}/policies/{2}'.format(self.version, cluster_id,
                                                     policy_id)
        resp, body = self.get(uri)

        return self.get_resp(resp, body)

    def cluster_collect(self, cluster, path):
        uri = '{0}/clusters/{1}/attrs/{2}'.format(self.version, cluster, path)
        resp, body = self.get(uri)

        return self.get_resp(resp, body)

    def wait_for_status(self, obj_type, obj_id, expected_status, timeout=None):
        if timeout is None:
            timeout = CONF.clustering.wait_timeout

        with timeutils.StopWatch(timeout) as timeout_watch:
            while timeout > 0:
                res = self.get_obj(obj_type, obj_id)
                if res['body']['status'] == expected_status:
                    return res
                time.sleep(5)
                timeout = timeout_watch.leftover(True)

        raise Exception('Timeout waiting for status.')

    def wait_for_delete(self, obj_type, obj_id, timeout=None):
        if timeout is None:
            timeout = CONF.clustering.wait_timeout

        with timeutils.StopWatch(timeout) as timeout_watch:
            while timeout > 0:
                try:
                    self.get_obj(obj_type, obj_id)
                except exceptions.NotFound:
                    return
                time.sleep(5)
                timeout = timeout_watch.leftover(True)

        raise Exception('Timeout waiting for deletion.')


class ClusteringFunctionalClient(ClusteringAPIClient):
    """This is the tempest client for Senlin functional test"""
    pass


class ClusteringIntegrationClient(ClusteringAPIClient):
    """This is the tempest client for Senlin integration test"""
    pass



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy

import mock
from oslo_context import context as oslo_ctx
import six

from senlin.common import context as senlin_ctx
from senlin.common import exception
from senlin.common import schema
from senlin.common import utils as common_utils
from senlin.engine import environment
from senlin.engine import parser
from senlin.objects import credential as co
from senlin.objects import profile as po
from senlin.profiles import base as pb
from senlin.profiles.os.nova import server as nova_server
from senlin.tests.unit.common import base
from senlin.tests.unit.common import utils


sample_profile = """
  type: os.dummy
  version: 1.0
  properties:
    key1: value1
    key2: 2
"""


class DummyProfile(pb.Profile):

    VERSION = '1.0'
    CONTEXT = 'context'

    properties_schema = {
        CONTEXT: schema.Map(
            'context data'
        ),
        'key1': schema.String(
            'first key',
            default='value1',
            updatable=True,
        ),
        'key2': schema.Integer(
            'second key',
            required=True,
            updatable=True,
        ),
        'key3': schema.String(
            'third key',
        ),
    }
    OPERATIONS = {
        'op1': schema.Operation(
            'Operation 1',
            schema={
                'param1': schema.StringParam(
                    'description of param1',
                )
            }
        )
    }

    def __init__(self, name, spec, **kwargs):
        super(DummyProfile, self).__init__(name, spec, **kwargs)


class TestProfileBase(base.SenlinTestCase):

    def setUp(self):
        super(TestProfileBase, self).setUp()
        self.ctx = utils.dummy_context(project='profile_test_project')
        g_env = environment.global_env()
        g_env.register_profile('os.dummy-1.0', DummyProfile)
        self.spec = parser.simple_parse(sample_profile)

    def _create_profile(self, name, pid=None, context=None):
        profile = pb.Profile(name, self.spec,
                             user=self.ctx.user,
                             project=self.ctx.project,
                             domain=self.ctx.domain,
                             context=context)
        if pid:
            profile.id = pid
            profile.context = context

        return profile

    @mock.patch.object(senlin_ctx, 'get_service_credentials')
    def test_init(self, mock_creds):
        mock_creds.return_value = {'foo': 'bar'}
        profile = self._create_profile('test-profile')

        self.assertIsNone(profile.id)
        self.assertEqual('test-profile', profile.name)
        self.assertEqual(self.spec, profile.spec)
        self.assertEqual('os.dummy', profile.type_name)
        self.assertEqual('1.0', profile.version)
        self.assertEqual('os.dummy-1.0', profile.type)
        self.assertEqual(self.ctx.user, profile.user)
        self.assertEqual(self.ctx.project, profile.project)
        self.assertEqual(self.ctx.domain, profile.domain)
        self.assertEqual({}, profile.metadata)
        self.assertIsNone(profile.created_at)
        self.assertIsNone(profile.updated_at)

        spec_data = profile.spec_data
        self.assertEqual('os.dummy', spec_data['type'])
        self.assertEqual('1.0', spec_data['version'])
        self.assertEqual('value1', spec_data['properties']['key1'])
        self.assertEqual(2, spec_data['properties']['key2'])
        self.assertEqual('value1', profile.properties['key1'])
        self.assertEqual(2, profile.properties['key2'])
        self.assertEqual({'foo': 'bar'}, profile.context)

        self.assertIsNone(profile._computeclient)
        self.assertIsNone(profile._networkclient)
        self.assertIsNone(profile._orchestrationclient)
        self.assertIsNone(profile._block_storageclient)

    @mock.patch.object(senlin_ctx, 'get_service_credentials')
    def test_init_with_context(self, mock_creds):
        mock_creds.return_value = {'foo': 'bar'}
        profile = self._create_profile('test-profile',
                                       pid='FAKE_ID', context={'bar': 'foo'})
        self.assertEqual({'bar': 'foo'}, profile.context)

    def test_init_bad_type(self):
        bad_spec = {
            'type': 'bad-type',
            'version': '1.0',
            'properties': '',
        }

        self.assertRaises(exception.ResourceNotFound,
                          pb.Profile,
                          'test-profile', bad_spec)

    def test_init_validation_error(self):
        bad_spec = copy.deepcopy(self.spec)
        del bad_spec['version']

        ex = self.assertRaises(exception.ESchema,
                               pb.Profile, 'test-profile', bad_spec)
        self.assertEqual("The 'version' key is missing from the provided "
                         "spec map.", six.text_type(ex))

    def test_from_object(self):
        obj = self._create_profile('test_profile_for_record')
        obj.store(self.ctx)
        profile = po.Profile.get(self.ctx, obj.id)

        result = pb.Profile._from_object(profile)

        self.assertEqual(profile.id, result.id)
        self.assertEqual(profile.name, result.name)
        self.assertEqual(profile.type, result.type)
        self.assertEqual(profile.user, result.user)
        self.assertEqual(profile.project, result.project)
        self.assertEqual(profile.domain, result.domain)
        self.assertEqual(profile.spec, result.spec)
        self.assertEqual(profile.metadata, result.metadata)
        self.assertEqual('value1', result.properties['key1'])
        self.assertEqual(2, result.properties['key2'])

        self.assertEqual(profile.created_at, result.created_at)
        self.assertEqual(profile.updated_at, result.updated_at)
        self.assertEqual(profile.context, result.context)

    def test_load_with_poect(self):
        obj = self._create_profile('test-profile-bb')
        profile_id = obj.store(self.ctx)
        profile = po.Profile.get(self.ctx, profile_id)

        result = pb.Profile.load(self.ctx, profile=profile)

        self.assertEqual(profile.id, result.id)

    def test_load_with_profile_id(self):
        obj = self._create_profile('test-profile-cc')
        profile_id = obj.store(self.ctx)

        result = pb.Profile.load(self.ctx, profile_id=profile_id)

        self.assertEqual(obj.id, result.id)

    def test_load_with_both(self):
        profile = self._create_profile('test1')
        profile.store(self.ctx)
        db_profile = po.Profile.get(self.ctx, profile.id)

        res = pb.Profile.load(self.ctx, profile=db_profile,
                              profile_id=profile.id)

        self.assertEqual(profile.id, res.id)

    @mock.patch.object(po.Profile, 'get')
    def test_load_not_found(self, mock_get):
        mock_get.return_value = None
        self.assertRaises(exception.ResourceNotFound,
                          pb.Profile.load,
                          self.ctx, profile_id='FAKE_ID')
        mock_get.assert_called_once_with(self.ctx, 'FAKE_ID',
                                         project_safe=True)

    @mock.patch.object(senlin_ctx, 'get_service_credentials')
    def test_create(self, mock_creds):
        mock_creds.return_value = {}
        res = pb.Profile.create(self.ctx, 'my_profile', self.spec)

        self.assertIsInstance(res, pb.Profile)

        obj = po.Profile.get(self.ctx, res.id)
        self.assertEqual('my_profile', obj.name)

    def test_create_profile_type_not_found(self):
        spec = copy.deepcopy(self.spec)
        spec['type'] = "bogus"
        ex = self.assertRaises(exception.InvalidSpec,
                               pb.Profile.create,
                               self.ctx, 'my_profile', spec)

        self.assertEqual("Failed in creating profile my_profile: The "
                         "profile_type 'bogus-1.0' could not be found.",
                         six.text_type(ex))

    @mock.patch.object(pb.Profile, 'validate')
    @mock.patch.object(senlin_ctx, 'get_service_credentials')
    def test_create_failed_validation(self, mock_creds, mock_validate):
        mock_creds.return_value = {}
        mock_validate.side_effect = exception.ESchema(message="Boom")

        ex = self.assertRaises(exception.InvalidSpec,
                               pb.Profile.create,
                               self.ctx, 'my_profile', self.spec)

        self.assertEqual("Failed in creating profile my_profile: "
                         "Boom", six.text_type(ex))

    @mock.patch.object(po.Profile, 'delete')
    def test_delete(self, mock_delete):
        res = pb.Profile.delete(self.ctx, 'FAKE_ID')
        self.assertIsNone(res)
        mock_delete.assert_called_once_with(self.ctx, 'FAKE_ID')

    @mock.patch.object(po.Profile, 'delete')
    def test_delete_busy(self, mock_delete):
        err = exception.EResourceBusy(type='profile', id='FAKE_ID')
        mock_delete.side_effect = err
        self.assertRaises(exception.EResourceBusy,
                          pb.Profile.delete,
                          self.ctx, 'FAKE_ID')
        mock_delete.assert_called_once_with(self.ctx, 'FAKE_ID')

    @mock.patch.object(po.Profile, 'delete')
    def test_delete_not_found(self, mock_delete):
        mock_delete.return_value = None
        result = pb.Profile.delete(self.ctx, 'BOGUS')
        self.assertIsNone(result)
        mock_delete.assert_called_once_with(self.ctx, 'BOGUS')

    @mock.patch.object(po.Profile, 'create')
    def test_store_for_create(self, mock_create):
        profile = self._create_profile('test-profile')
        self.assertIsNone(profile.id)
        self.assertIsNone(profile.created_at)

        mock_create.return_value = mock.Mock(id='FAKE_ID')

        profile_id = profile.store(self.ctx)

        mock_create.assert_called_once_with(
            self.ctx,
            {
                'name': profile.name,
                'type': profile.type,
                'context': profile.context,
                'spec': profile.spec,
                'user': profile.user,
                'project': profile.project,
                'domain': profile.domain,
                'meta_data': profile.metadata,
                'created_at': mock.ANY,
            }
        )
        self.assertEqual('FAKE_ID', profile_id)
        self.assertIsNotNone(profile.created_at)

    @mock.patch.object(po.Profile, 'update')
    def test_store_for_update(self, mock_update):
        profile = self._create_profile('test-profile')
        self.assertIsNone(profile.id)
        self.assertIsNone(profile.updated_at)
        profile.id = 'FAKE_ID'

        profile_id = profile.store(self.ctx)
        self.assertEqual('FAKE_ID', profile.id)
        mock_update.assert_called_once_with(
            self.ctx,
            'FAKE_ID',
            {
                'name': profile.name,
                'type': profile.type,
                'context': profile.context,
                'spec': profile.spec,
                'user': profile.user,
                'project': profile.project,
                'domain': profile.domain,
                'meta_data': profile.metadata,
                'updated_at': mock.ANY,
            }
        )
        self.assertIsNotNone(profile.updated_at)
        self.assertEqual('FAKE_ID', profile_id)

    @mock.patch.object(pb.Profile, 'load')
    def test_create_object(self, mock_load):
        profile = mock.Mock()
        mock_load.return_value = profile
        obj = mock.Mock()
        obj.profile_id = 'FAKE_ID'

        res = pb.Profile.create_object(self.ctx, obj)

        mock_load.assert_called_once_with(self.ctx, profile_id='FAKE_ID')
        profile.do_create.assert_called_once_with(obj)
        res_obj = profile.do_create.return_value
        self.assertEqual(res_obj, res)

    @mock.patch.object(pb.Profile, 'load')
    def test_check_object(self, mock_load):
        profile = mock.Mock()
        mock_load.return_value = profile
        obj = mock.Mock()
        obj.profile_id = 'FAKE_ID'

        res = pb.Profile.check_object(self.ctx, obj)

        mock_load.assert_called_once_with(self.ctx, profile_id='FAKE_ID')
        profile.do_check.assert_called_once_with(obj)
        res_obj = profile.do_check.return_value
        self.assertEqual(res_obj, res)

    @mock.patch.object(pb.Profile, 'load')
    def test_delete_object(self, mock_load):
        profile = mock.Mock()
        mock_load.return_value = profile
        obj = mock.Mock()
        obj.profile_id = 'FAKE_ID'

        res = pb.Profile.delete_object(self.ctx, obj)

        mock_load.assert_called_once_with(self.ctx, profile_id='FAKE_ID')
        profile.do_delete.assert_called_once_with(obj)
        res_obj = profile.do_delete.return_value
        self.assertEqual(res_obj, res)

    @mock.patch.object(pb.Profile, 'load')
    def test_check_object_exception_return_value(self, mock_load):
        profile = pb.Profile
        profile.load(self.ctx).do_check = mock.Mock(
            side_effect=exception.InternalError(code=400, message='BAD'))
        obj = mock_load

        res = profile.check_object(self.ctx, obj)

        profile.load(self.ctx).do_check.assert_called_once_with(obj)
        self.assertFalse(res)

    @mock.patch.object(pb.Profile, 'load')
    def test_update_object_with_profile(self, mock_load):
        old_profile = mock.Mock()
        new_profile = mock.Mock()

        mock_load.side_effect = [old_profile, new_profile]
        obj = mock.Mock()
        obj.profile_id = 'OLD_ID'

        res = pb.Profile.update_object(self.ctx, obj,
                                       new_profile_id='NEW_ID', foo='bar')

        mock_load.assert_has_calls([
            mock.call(self.ctx, profile_id='OLD_ID'),
            mock.call(self.ctx, profile_id='NEW_ID'),
        ])

        old_profile.do_update.assert_called_once_with(obj, new_profile,
                                                      foo='bar')
        res_obj = old_profile.do_update.return_value
        self.assertEqual(res_obj, res)

    @mock.patch.object(pb.Profile, 'load')
    def test_update_object_without_profile(self, mock_load):
        profile = mock.Mock()

        mock_load.return_value = profile
        obj = mock.Mock()
        obj.profile_id = 'FAKE_ID'

        res = pb.Profile.update_object(self.ctx, obj, foo='bar', zoo='car')

        mock_load.assert_called_once_with(self.ctx, profile_id='FAKE_ID')
        profile.do_update.assert_called_once_with(obj, None,
                                                  foo='bar', zoo='car')
        res_obj = profile.do_update.return_value
        self.assertEqual(res_obj, res)

    @mock.patch.object(pb.Profile, 'load')
    def test_recover_object(self, mock_load):
        profile = mock.Mock()
        mock_load.return_value = profile
        obj = mock.Mock()
        obj.profile_id = 'FAKE_ID'

        res = pb.Profile.recover_object(self.ctx, obj, foo='bar', zoo='car')

        mock_load.assert_called_once_with(self.ctx, profile_id='FAKE_ID')
        profile.do_recover.assert_called_once_with(obj, foo='bar', zoo='car')
        res_obj = profile.do_recover.return_value
        self.assertEqual(res_obj, res)

    @mock.patch.object(pb.Profile, 'load')
    def test_get_details(self, mock_load):
        profile = mock.Mock()
        mock_load.return_value = profile
        obj = mock.Mock()
        obj.profile_id = 'FAKE_ID'

        res = pb.Profile.get_details(self.ctx, obj)

        mock_load.assert_called_once_with(self.ctx, profile_id='FAKE_ID')
        profile.do_get_details.assert_called_once_with(obj)
        res_obj = profile.do_get_details.return_value
        self.assertEqual(res_obj, res)

    def test_get_schema(self):
        expected = {
            'context': {
                'description': 'context data',
                'required': False,
                'updatable': False,
                'type': 'Map'
            },
            'key1': {
                'default': 'value1',
                'description': 'first key',
                'required': False,
                'updatable': True,
                'type': 'String',
            },
            'key2': {
                'description': 'second key',
                'required': True,
                'updatable': True,
                'type': 'Integer'
            },
            'key3': {
                'description': 'third key',
                'required': False,
                'updatable': False,
                'type': 'String'
            },
        }

        actual = DummyProfile.get_schema()
        self.assertEqual(expected, actual)

    def test_get_ops(self):
        expected = {
            'op1': {
                'description': 'Operation 1',
                'parameters': {
                    'param1': {
                        'type': 'String',
                        'required': False,
                        'description': 'description of param1',
                    }
                }
            },
        }

        actual = DummyProfile.get_ops()
        self.assertEqual(expected, actual)

    @mock.patch.object(nova_server.ServerProfile, 'do_adopt')
    def test_adopt_node(self, mock_adopt):
        obj = mock.Mock()

        res = pb.Profile.adopt_node(self.ctx, obj, "os.nova.server-1.0",
                                    overrides=None, snapshot=False)

        mock_adopt.assert_called_once_with(obj, overrides=None, snapshot=False)
        res_obj = mock_adopt.return_value
        self.assertEqual(res_obj, res)

    @mock.patch.object(pb.Profile, 'load')
    def test_join_cluster(self, mock_load):
        profile = mock.Mock()
        mock_load.return_value = profile
        obj = mock.Mock()
        obj.profile_id = 'FAKE_ID'

        res = pb.Profile.join_cluster(self.ctx, obj, 'CLUSTER_ID')

        mock_load.assert_called_once_with(self.ctx, profile_id='FAKE_ID')
        profile.do_join.assert_called_once_with(obj, 'CLUSTER_ID')
        res_obj = profile.do_join.return_value
        self.assertEqual(res_obj, res)

    @mock.patch.object(pb.Profile, 'load')
    def test_leave_cluster(self, mock_load):
        profile = mock.Mock()
        mock_load.return_value = profile
        obj = mock.Mock()
        obj.profile_id = 'FAKE_ID'

        res = pb.Profile.leave_cluster(self.ctx, obj)

        mock_load.assert_called_once_with(self.ctx, profile_id='FAKE_ID')
        profile.do_leave.assert_called_once_with(obj)
        res_obj = profile.do_leave.return_value
        self.assertEqual(res_obj, res)

    def test_validate_without_properties(self):
        profile = self._create_profile('test_profile')

        profile.do_validate = mock.Mock()

        profile.validate()

        profile.do_validate.assert_not_called()

    def test_validate_with_properties(self):
        profile = self._create_profile('test_profile')

        profile.do_validate = mock.Mock()

        profile.validate(validate_props=True)

        profile.do_validate.assert_called_once_with(obj=profile)

    def test_validate_bad_context(self):
        spec = {
            "type": "os.dummy",
            "version": "1.0",
            "properties": {
                "context": {
                    "foo": "bar"
                },
                "key1": "value1",
                "key2": 2,
            }
        }
        profile = DummyProfile("p-bad-ctx", spec, user=self.ctx.user,
                               project=self.ctx.project,
                               domain=self.ctx.domain)

        self.assertRaises(exception.ESchema, profile.validate)

    @mock.patch.object(senlin_ctx, 'get_service_credentials')
    def test__init_context(self, mock_creds):
        fake_ctx = mock.Mock()
        mock_creds.return_value = fake_ctx

        # _init_context() is called from __init__
        self._create_profile('test-profile')

        # cannot determin the result in this case, we only test none or not
        fake_ctx.pop.assert_has_calls([
            mock.call('project_name', None),
            mock.call('project_domain_name', None),
        ])
        mock_creds.assert_called_once_with()

    @mock.patch.object(senlin_ctx, 'get_service_credentials')
    def test__init_context_for_real(self, mock_creds):
        fake_ctx = {
            'project_name': 'this project',
            'project_domain_name': 'this domain',
            'auth_url': 'some url',
            'user_id': 'fake_user',
            'foo': 'bar',
        }
        mock_creds.return_value = fake_ctx

        # _init_context() is called from __init__
        profile = self._create_profile('test-profile')

        mock_creds.assert_called_once_with()
        expected = {
            'auth_url': 'some url',
            'user_id': 'fake_user',
            'foo': 'bar',
        }
        self.assertEqual(expected, profile.context)

    @mock.patch.object(senlin_ctx, 'get_service_credentials')
    def test__init_context_for_real_with_data(self, mock_creds):
        fake_ctx = {
            'project_name': 'this project',
            'project_domain_name': 'this domain',
            'auth_url': 'some url',
            'user_id': 'fake_user',
            'foo': 'bar',
        }
        mock_creds.return_value = fake_ctx
        self.spec['properties']['context'] = {
            'region_name': 'region_dist'
        }

        # _init_context() is called from __init__
        profile = self._create_profile('test-profile')

        mock_creds.assert_called_once_with(region_name='region_dist')
        expected = {
            'auth_url': 'some url',
            'user_id': 'fake_user',
            'foo': 'bar',
        }
        self.assertEqual(expected, profile.context)

    @mock.patch.object(co.Credential, 'get')
    @mock.patch.object(oslo_ctx, 'get_current')
    def test__build_conn_params(self, mock_current, mock_get):
        profile = self._create_profile('test-profile')
        profile.context = {'foo': 'bar'}
        fake_cred = mock.Mock(cred={'openstack': {'trust': 'TRUST_ID'}})
        mock_get.return_value = fake_cred
        fake_ctx = mock.Mock()
        mock_current.return_value = fake_ctx

        user = 'FAKE_USER'
        project = 'FAKE_PROJECT'

        res = profile._build_conn_params(user, project)
        expected = {
            'foo': 'bar',
            'trust_id': 'TRUST_ID',
        }
        self.assertEqual(expected, res)
        mock_current.assert_called_once_with()
        mock_get.assert_called_once_with(fake_ctx, 'FAKE_USER', 'FAKE_PROJECT')

    @mock.patch.object(co.Credential, 'get')
    @mock.patch.object(oslo_ctx, 'get_current')
    def test__build_conn_params_trust_not_found(self, mock_current, mock_get):
        profile = self._create_profile('test-profile')
        mock_get.return_value = None
        fake_ctx = mock.Mock()
        mock_current.return_value = fake_ctx

        self.assertRaises(exception.TrustNotFound,
                          profile._build_conn_params,
                          'FAKE_USER', 'FAKE_PROJECT')

        mock_current.assert_called_once_with()
        mock_get.assert_called_once_with(fake_ctx, 'FAKE_USER', 'FAKE_PROJECT')

    @mock.patch.object(pb.Profile, '_build_conn_params')
    @mock.patch("senlin.drivers.base.SenlinDriver")
    def test_compute(self, mock_senlindriver, mock_params):
        obj = mock.Mock()
        sd = mock.Mock()
        cc = mock.Mock()
        sd.compute.return_value = cc
        mock_senlindriver.return_value = sd
        fake_params = mock.Mock()
        mock_params.return_value = fake_params
        profile = self._create_profile('test-profile')

        res = profile.compute(obj)

        self.assertEqual(cc, res)
        self.assertEqual(cc, profile._computeclient)
        mock_params.assert_called_once_with(obj.user, obj.project)
        sd.compute.assert_called_once_with(fake_params)

    def test_compute_with_cache(self):
        cc = mock.Mock()
        profile = self._create_profile('test-profile')
        profile._computeclient = cc

        res = profile.compute(mock.Mock())

        self.assertEqual(cc, res)

    @mock.patch.object(pb.Profile, '_build_conn_params')
    @mock.patch("senlin.drivers.base.SenlinDriver")
    def test_neutron_client(self, mock_senlindriver, mock_params):
        obj = mock.Mock()
        sd = mock.Mock()
        nc = mock.Mock()
        sd.network.return_value = nc
        mock_senlindriver.return_value = sd
        fake_params = mock.Mock()
        mock_params.return_value = fake_params
        profile = self._create_profile('test-profile')

        res = profile.network(obj)

        self.assertEqual(nc, res)
        self.assertEqual(nc, profile._networkclient)
        mock_params.assert_called_once_with(obj.user, obj.project)
        sd.network.assert_called_once_with(fake_params)

    @mock.patch.object(pb.Profile, '_build_conn_params')
    @mock.patch("senlin.drivers.base.SenlinDriver")
    def test_cinder_client(self, mock_senlindriver, mock_params):
        obj = mock.Mock()
        sd = mock.Mock()
        nc = mock.Mock()
        sd.block_storage.return_value = nc
        mock_senlindriver.return_value = sd
        fake_params = mock.Mock()
        mock_params.return_value = fake_params
        profile = self._create_profile('test-profile')

        res = profile.block_storage(obj)

        self.assertEqual(nc, res)
        self.assertEqual(nc, profile._block_storageclient)
        mock_params.assert_called_once_with(obj.user, obj.project)
        sd.block_storage.assert_called_once_with(fake_params)

    def test_interface_methods(self):
        profile = self._create_profile('test-profile')

        self.assertRaises(NotImplementedError, profile.do_create, mock.Mock())
        self.assertRaises(NotImplementedError, profile.do_delete, mock.Mock())
        self.assertTrue(profile.do_update(mock.Mock(), mock.Mock()))
        self.assertTrue(profile.do_check(mock.Mock()))
        self.assertEqual({}, profile.do_get_details(mock.Mock()))
        self.assertTrue(profile.do_join(mock.Mock(), mock.Mock()))
        self.assertTrue(profile.do_leave(mock.Mock()))
        self.assertTrue(profile.do_validate(mock.Mock()))

    def test_do_recover_default(self):
        profile = self._create_profile('test-profile')
        self.patchobject(profile, 'do_create', return_value=True)
        self.patchobject(profile, 'do_delete', return_value=True)

        res = profile.do_recover(mock.Mock())
        self.assertTrue(res)

        res = profile.do_recover(mock.Mock(), operation=[{'name': 'bar'}])
        self.assertFalse(res)

    def test_do_recover_with_fencing(self):
        profile = self._create_profile('test-profile')
        self.patchobject(profile, 'do_create', return_value=True)
        self.patchobject(profile, 'do_delete', return_value=True)
        obj = mock.Mock()

        res = profile.do_recover(obj, ignore_missing=True,
                                 params={"fence_compute": True})

        self.assertTrue(res)
        profile.do_delete.assert_called_once_with(obj, force=True)
        profile.do_create.assert_called_once_with(obj)

    def test_do_recover_with_recreate_succeeded(self):
        profile = self._create_profile('test-profile')
        self.patchobject(profile, 'do_delete', return_value=True)
        self.patchobject(profile, 'do_create', return_value=True)
        operation = [{"name": "RECREATE"}]
        res = profile.do_recover(mock.Mock(), operation=operation)

        self.assertTrue(res)

    def test_do_recover_with_recreate_failed_delete(self):
        profile = self._create_profile('test-profile')
        err = exception.EResourceDeletion(type='STACK', id='ID',
                                          message='BANG')
        self.patchobject(profile, 'do_delete', side_effect=err)
        operation = [{"name": "RECREATE"}]

        ex = self.assertRaises(exception.EResourceOperation,
                               profile.do_recover,
                               mock.Mock(id='NODE_ID'), operation=operation)
        self.assertEqual("Failed in recovering node 'NODE_ID': "
                         "Failed in deleting STACK 'ID': BANG.",
                         six.text_type(ex))

    def test_do_recover_with_recreate_failed_create(self):
        profile = self._create_profile('test-profile')
        self.patchobject(profile, 'do_delete', return_value=True)
        err = exception.EResourceCreation(type='STACK', message='BANNG')
        self.patchobject(profile, 'do_create', side_effect=err)
        operation = [{"name": "RECREATE"}]

        ex = self.assertRaises(exception.EResourceOperation,
                               profile.do_recover,
                               mock.Mock(id='NODE_ID'), operation=operation)

        msg = ("Failed in recovering node 'NODE_ID': Failed in creating "
               "STACK: BANNG.")
        self.assertEqual(msg, six.text_type(ex))

    def test_to_dict(self):
        profile = self._create_profile('test-profile')
        # simulate a store()
        profile.id = 'FAKE_ID'
        expected = {
            'id': 'FAKE_ID',
            'name': profile.name,
            'type': profile.type,
            'user': profile.user,
            'project': profile.project,
            'domain': profile.domain,
            'spec': profile.spec,
            'metadata': profile.metadata,
            'created_at': common_utils.isotime(profile.created_at),
            'updated_at': None,
        }

        result = profile.to_dict()
        self.assertEqual(expected, result)

    def test_validate_for_update_succeeded(self):
        profile = self._create_profile('test-profile')

        # Properties are updatable
        new_spec = copy.deepcopy(self.spec)
        new_spec['properties']['key1'] = 'new_v1'
        new_spec['properties']['key2'] = 3
        new_profile = pb.Profile('new-profile', new_spec,
                                 user=self.ctx.user,
                                 project=self.ctx.project,
                                 domain=self.ctx.domain,
                                 context=None)
        res = profile.validate_for_update(new_profile)
        self.assertTrue(res)

    def test_validate_for_update_failed(self):
        profile = self._create_profile('test-profile')

        # Property is not updatable
        new_spec = copy.deepcopy(self.spec)
        new_spec['properties']['key3'] = 'new_v3'
        new_profile = pb.Profile('new-profile', new_spec,
                                 user=self.ctx.user,
                                 project=self.ctx.project,
                                 domain=self.ctx.domain,
                                 context=None)

        res = profile.validate_for_update(new_profile)
        self.assertFalse(res)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from senlin.tests.tempest.api import base
from senlin.tests.tempest.common import utils


class TestProfileTypeList(base.BaseSenlinAPITest):

    @utils.api_microversion('1.4')
    @decorators.idempotent_id('fa0cf9e3-5b75-4d4d-9a0f-1748772b65d3')
    def test_profile_type_list(self):
        res = self.client.list_objs('profile-types')

        # Verify resp of profile type list API
        self.assertEqual(200, res['status'])
        self.assertIsNotNone(res['body'])
        profile_types = res['body']
        expected_profile_types = [
            {'name': 'os.nova.server-1.0'},
            {'name': 'os.heat.stack-1.0'},
            {'name': 'container.dockerinc.docker-1.0'}
        ]
        for profile_type in expected_profile_types:
            self.assertIn(profile_type, profile_types)

    @utils.api_microversion('1.5')
    @decorators.idempotent_id('778d41df-0ce0-421f-98e5-2efdcec6d995')
    def test_profile_type_list_v1_5(self):
        res = self.client.list_objs('profile-types')

        # Verify resp of profile type list API
        self.assertEqual(200, res['status'])
        self.assertIsNotNone(res['body'])
        profile_types = res['body']
        expected_names = [
            'os.nova.server',
            'os.heat.stack',
            'container.dockerinc.docker'
        ]
        for t in profile_types:
            self.assertIn(t['name'], expected_names)
            self.assertIsNotNone(t['support_status'])
            self.assertIsNotNone(t['version'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
from oslo_config import cfg
import six

from senlin.common import consts
from senlin.common import exception
from senlin.engine import cluster as cm
from senlin.engine import cluster_policy as cpm
from senlin.engine import health_manager
from senlin.engine import node as node_mod
from senlin.objects import cluster as co
from senlin.objects import cluster_policy as cpo
from senlin.objects import node as no
from senlin.policies import base as pcb
from senlin.profiles import base as pfb
from senlin.tests.unit.common import base
from senlin.tests.unit.common import utils

PROFILE_ID = 'aa5f86b8-e52b-4f2b-828a-4c14c770938d'
CLUSTER_ID = '60efdaa1-06c2-4fcf-ae44-17a2d85ff3ea'
POLICY_ID = '2c5139a6-24ba-4a6f-bd53-a268f61536de'


class TestCluster(base.SenlinTestCase):

    def setUp(self):
        super(TestCluster, self).setUp()
        self.context = utils.dummy_context(project='cluster_test_project')

    def test_init(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)

        self.assertIsNone(cluster.id)
        self.assertEqual('test-cluster', cluster.name)
        self.assertEqual(PROFILE_ID, cluster.profile_id)
        self.assertEqual('', cluster.user)
        self.assertEqual('', cluster.project)
        self.assertEqual('', cluster.domain)

        self.assertIsNone(cluster.init_at)
        self.assertIsNone(cluster.created_at)
        self.assertIsNone(cluster.updated_at)

        self.assertEqual(0, cluster.min_size)
        self.assertEqual(-1, cluster.max_size)
        self.assertEqual(0, cluster.desired_capacity)
        self.assertEqual(1, cluster.next_index)
        self.assertEqual(cfg.CONF.default_action_timeout, cluster.timeout)
        self.assertEqual('INIT', cluster.status)
        self.assertEqual('Initializing', cluster.status_reason)
        self.assertEqual({}, cluster.data)
        self.assertEqual({}, cluster.metadata)
        self.assertEqual({}, cluster.dependents)
        self.assertEqual({}, cluster.config)
        self.assertEqual({'profile': None, 'nodes': [], 'policies': []},
                         cluster.rt)

    def test_init_with_none(self):
        kwargs = {
            'min_size': None,
            'max_size': None,
            'metadata': None
        }
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID, **kwargs)
        self.assertEqual(0, cluster.min_size)
        self.assertEqual(-1, cluster.max_size)
        self.assertEqual({}, cluster.metadata)

    @mock.patch.object(cm.Cluster, '_load_runtime_data')
    def test_init_with_context(self, mock_load):
        cm.Cluster('test-cluster', 0, PROFILE_ID, context=self.context)
        mock_load.assert_called_once_with(self.context)

    @mock.patch.object(cpo.ClusterPolicy, 'get_all')
    @mock.patch.object(pcb.Policy, 'load')
    @mock.patch.object(pfb.Profile, 'load')
    @mock.patch.object(no.Node, 'get_all_by_cluster')
    def test__load_runtime_data(self, mock_nodes, mock_profile, mock_policy,
                                mock_pb):
        x_binding = mock.Mock()
        x_binding.policy_id = POLICY_ID
        mock_pb.return_value = [x_binding]
        x_policy = mock.Mock()
        mock_policy.return_value = x_policy
        x_profile = mock.Mock()
        mock_profile.return_value = x_profile
        x_node_1 = mock.Mock()
        x_node_2 = mock.Mock()
        mock_nodes.return_value = [x_node_1, x_node_2]

        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        cluster.id = CLUSTER_ID

        cluster._load_runtime_data(self.context)

        rt = cluster.rt
        self.assertEqual(x_profile, rt['profile'])
        self.assertEqual([x_node_1, x_node_2], rt['nodes'])
        self.assertEqual(2, len(rt['nodes']))
        self.assertIsInstance(rt['nodes'], list)
        self.assertEqual([x_policy], rt['policies'])

        mock_pb.assert_called_once_with(self.context, CLUSTER_ID)
        mock_policy.assert_called_once_with(self.context,
                                            POLICY_ID,
                                            project_safe=False)
        mock_profile.assert_called_once_with(self.context,
                                             profile_id=PROFILE_ID,
                                             project_safe=False)
        mock_nodes.assert_called_once_with(self.context, CLUSTER_ID)

    def test__load_runtime_data_id_is_none(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)

        cluster._load_runtime_data(self.context)

        rt = cluster.rt
        self.assertIsNone(rt['profile'])
        self.assertEqual([], rt['nodes'])
        self.assertEqual(0, len(rt['nodes']))
        self.assertIsInstance(rt['nodes'], list)
        self.assertEqual([], rt['policies'])

    def test_store_for_create(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID,
                             user=self.context.user,
                             project=self.context.project)
        mock_load = self.patchobject(cluster, '_load_runtime_data')
        self.assertIsNone(cluster.id)

        cluster_id = cluster.store(self.context)
        self.assertIsNotNone(cluster_id)
        mock_load.assert_called_once_with(self.context)

        result = co.Cluster.get(self.context, cluster_id=cluster_id)

        self.assertIsNotNone(result)
        self.assertEqual('test-cluster', result.name)
        self.assertEqual(PROFILE_ID, result.profile_id)
        self.assertEqual(self.context.user, result.user)
        self.assertEqual(self.context.project, result.project)
        self.assertEqual(self.context.domain, result.domain)

        self.assertIsNotNone(result.init_at)
        self.assertIsNone(result.created_at)
        self.assertIsNone(result.updated_at)

        self.assertEqual(0, result.min_size)
        self.assertEqual(-1, result.max_size)
        self.assertEqual(0, result.desired_capacity)
        self.assertEqual(1, result.next_index)
        self.assertEqual(cfg.CONF.default_action_timeout, result.timeout)
        self.assertEqual('INIT', result.status)
        self.assertEqual('Initializing', result.status_reason)
        self.assertEqual({}, result.data)
        self.assertEqual({}, result.metadata)

    def test_store_for_update(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID,
                             user=self.context.user,
                             project=self.context.project)
        mock_load = self.patchobject(cluster, '_load_runtime_data')
        self.assertIsNone(cluster.id)

        cluster_id = cluster.store(self.context)

        self.assertIsNotNone(cluster_id)
        mock_load.assert_called_once_with(self.context)

        # do an update
        cluster.name = 'test-cluster-1'

        cluster.min_size = 1
        cluster.max_size = 3
        cluster.desired_capacity = 2
        cluster.timeout = 120
        cluster.data = {'FOO': 'BAR'}
        cluster.metadata = {'KEY': 'VALUE'}
        cluster.config = {'KEY': 'VALUE'}

        new_id = cluster.store(self.context)
        self.assertEqual(cluster_id, new_id)

        result = co.Cluster.get(self.context, cluster_id)
        self.assertIsNotNone(result)

        self.assertEqual('test-cluster-1', result.name)
        self.assertEqual(self.context.user, result.user)
        self.assertEqual(self.context.project, result.project)

        self.assertEqual(1, result.min_size)
        self.assertEqual(3, result.max_size)
        self.assertEqual(2, result.desired_capacity)

        self.assertEqual(120, result.timeout)
        self.assertEqual({'FOO': 'BAR'}, result.data)
        self.assertEqual({'KEY': 'VALUE'}, result.metadata)
        self.assertEqual({'KEY': 'VALUE'}, result.config)

    @mock.patch.object(cm.Cluster, '_from_object')
    def test_load_via_db_object(self, mock_init):
        x_obj = mock.Mock()

        result = cm.Cluster.load(self.context, dbcluster=x_obj)

        self.assertEqual(mock_init.return_value, result)
        mock_init.assert_called_once_with(self.context, x_obj)

    @mock.patch.object(co.Cluster, 'get')
    @mock.patch.object(cm.Cluster, '_from_object')
    def test_load_via_cluster_id(self, mock_init, mock_get):
        x_obj = mock.Mock()
        mock_get.return_value = x_obj

        result = cm.Cluster.load(self.context, cluster_id=CLUSTER_ID)

        self.assertEqual(mock_init.return_value, result)
        mock_get.assert_called_once_with(self.context, CLUSTER_ID,
                                         project_safe=True)
        mock_init.assert_called_once_with(self.context, x_obj)

    @mock.patch.object(co.Cluster, 'get')
    def test_load_not_found(self, mock_get):
        mock_get.return_value = None
        ex = self.assertRaises(exception.ResourceNotFound,
                               cm.Cluster.load,
                               self.context, cluster_id=CLUSTER_ID)
        self.assertEqual("The cluster '%s' could not be found." % CLUSTER_ID,
                         six.text_type(ex))
        mock_get.assert_called_once_with(self.context, CLUSTER_ID,
                                         project_safe=True)

    @mock.patch.object(cm.Cluster, '_from_object')
    @mock.patch.object(co.Cluster, 'get_all')
    def test_load_all(self, mock_get, mock_init):
        x_obj_1 = mock.Mock()
        x_obj_2 = mock.Mock()
        mock_get.return_value = [x_obj_1, x_obj_2]

        x_cluster_1 = mock.Mock()
        x_cluster_2 = mock.Mock()
        mock_init.side_effect = [x_cluster_1, x_cluster_2]

        result = cm.Cluster.load_all(self.context)

        self.assertEqual([x_cluster_1, x_cluster_2], [c for c in result])
        mock_get.assert_called_once_with(self.context,
                                         limit=None, marker=None,
                                         sort=None, filters=None,
                                         project_safe=True)
        mock_init.assert_has_calls([
            mock.call(self.context, x_obj_1),
            mock.call(self.context, x_obj_2)])

    @mock.patch.object(co.Cluster, 'update')
    def test_set_status_for_create(self, mock_update):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID,
                             id=CLUSTER_ID, status='CREATING')

        cluster.set_status(self.context, consts.CS_ACTIVE, 'Cluster created')

        self.assertEqual(consts.CS_ACTIVE, cluster.status)
        self.assertEqual('Cluster created', cluster.status_reason)
        self.assertIsNotNone(cluster.created_at)
        self.assertIsNone(cluster.updated_at)

        mock_update.assert_called_once_with(
            self.context, CLUSTER_ID,
            {
                'created_at': mock.ANY,
                'status': consts.CS_ACTIVE,
                'status_reason': 'Cluster created'
            }
        )

    @mock.patch.object(co.Cluster, 'update')
    def test_set_status_for_update(self, mock_update):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID,
                             id=CLUSTER_ID, status='UPDATING')

        cluster.set_status(self.context, consts.CS_ACTIVE, 'Cluster updated')

        self.assertEqual(consts.CS_ACTIVE, cluster.status)
        self.assertEqual('Cluster updated', cluster.status_reason)
        self.assertIsNotNone(cluster.updated_at)

    @mock.patch.object(co.Cluster, 'update')
    def test_set_status_for_resize(self, mock_update):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID,
                             id=CLUSTER_ID, status='RESIZING')

        cluster.set_status(self.context, consts.CS_ACTIVE, 'Cluster resized')

        self.assertEqual(consts.CS_ACTIVE, cluster.status)
        self.assertEqual('Cluster resized', cluster.status_reason)
        self.assertIsNotNone(cluster.updated_at)

    @mock.patch.object(pfb.Profile, 'load')
    @mock.patch.object(co.Cluster, 'update')
    def test_set_status_for_update_with_profile(self, mock_update,
                                                mock_load):
        x_profile = mock.Mock()
        mock_load.return_value = x_profile
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID, id=CLUSTER_ID,
                             status='UPDATING')

        new_profile_id = 'a64f0b03-4b77-49d5-89e0-7bcc77c4ce67'
        cluster.set_status(self.context, consts.CS_ACTIVE, 'Cluster updated',
                           profile_id=new_profile_id)

        self.assertEqual(consts.CS_ACTIVE, cluster.status)
        self.assertEqual('Cluster updated', cluster.status_reason)
        self.assertIsNotNone(cluster.updated_at)
        self.assertEqual(x_profile, cluster.rt['profile'])
        self.assertEqual(new_profile_id, cluster.profile_id)
        mock_load.assert_called_once_with(self.context,
                                          profile_id=new_profile_id)
        mock_update.assert_called_once_with(
            self.context, CLUSTER_ID,
            {
                'status': consts.CS_ACTIVE,
                'status_reason': 'Cluster updated',
                'profile_id': new_profile_id,
                'updated_at': mock.ANY,
            }
        )

    @mock.patch.object(co.Cluster, 'update')
    def test_set_status_without_reason(self, mock_update):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID, id=CLUSTER_ID,
                             status='UPDATING',
                             status_reason='Update in progress')

        cluster.set_status(self.context, consts.CS_WARNING)

        self.assertEqual(consts.CS_WARNING, cluster.status)
        self.assertEqual('Update in progress', cluster.status_reason)
        mock_update.assert_called_once_with(self.context, CLUSTER_ID,
                                            {'status': consts.CS_WARNING})

    @mock.patch.object(pfb.Profile, "create_cluster_object")
    def test_do_create(self, mock_create_cluster):
        mock_create_cluster.return_value = None

        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        mock_status = self.patchobject(cluster, 'set_status')

        res = cluster.do_create(self.context)

        self.assertTrue(res)
        mock_status.assert_called_once_with(
            self.context, consts.CS_CREATING, 'Creation in progress')

    def test_do_create_wrong_status(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        cluster.status = consts.CS_ACTIVE

        res = cluster.do_create(self.context)

        self.assertFalse(res)

    @mock.patch.object(pfb.Profile, "delete_cluster_object")
    @mock.patch.object(co.Cluster, 'delete')
    def test_do_delete(self, mock_delete, mock_delete_cluster):
        mock_delete.return_value = None
        mock_delete_cluster.return_value = None

        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        cluster.id = CLUSTER_ID
        mock_status = self.patchobject(cluster, 'set_status')

        res = cluster.do_delete(self.context)

        mock_delete.assert_called_once_with(self.context, CLUSTER_ID)
        self.assertTrue(res)
        mock_status.assert_called_once_with(
            self.context, consts.CS_DELETING, 'Deletion in progress')

    def test_do_update(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        mock_status = self.patchobject(cluster, 'set_status')

        res = cluster.do_update(self.context)

        mock_status.assert_called_once_with(self.context, consts.CS_UPDATING,
                                            'Update in progress')
        self.assertTrue(res)

    def test_do_check(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        mock_status = self.patchobject(cluster, 'set_status')

        res = cluster.do_check(self.context)

        mock_status.assert_called_once_with(self.context, consts.CS_CHECKING,
                                            'Check in progress')
        self.assertTrue(res)

    def test_do_recover(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        mock_status = self.patchobject(cluster, 'set_status')

        res = cluster.do_recover(self.context)

        mock_status.assert_called_once_with(self.context, consts.CS_RECOVERING,
                                            'Recovery in progress')
        self.assertTrue(res)

    def test_do_operation(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        mock_status = self.patchobject(cluster, 'set_status')

        res = cluster.do_operation(self.context, operation='dance')

        mock_status.assert_called_once_with(self.context, consts.CS_OPERATING,
                                            'Operation dance in progress')
        self.assertTrue(res)

    def test_nodes_property(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        self.assertEqual([], cluster.nodes)

        # with nodes
        node1 = mock.Mock()
        node2 = mock.Mock()
        cluster.rt['nodes'] = [node1, node2]

        self.assertEqual([node1, node2], cluster.nodes)

    def test_policies_property(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        self.assertEqual([], cluster.policies)

        # with policies attached
        policy1 = mock.Mock()
        policy2 = mock.Mock()
        cluster.rt['policies'] = [policy1, policy2]
        self.assertEqual([policy1, policy2], cluster.policies)

    def test_add_node(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        self.assertEqual([], cluster.nodes)

        # add one node
        node = mock.Mock()
        cluster.add_node(node)
        self.assertEqual([node], cluster.nodes)

        # add another node
        another_node = mock.Mock()
        cluster.add_node(another_node)
        self.assertEqual([node, another_node], cluster.nodes)

    def test_remove_node(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        self.assertEqual([], cluster.nodes)

        # remove from empty list should be okay
        res = cluster.remove_node('BOGUS')
        self.assertIsNone(res)

        # add one node
        node1 = mock.Mock()
        node1.id = '62d52dd6-5f83-4340-b079-349da2f9ffd9'
        cluster.add_node(node1)
        self.assertEqual([node1], cluster.nodes)

        # remove non-existent node should be okay
        node2 = mock.Mock()
        node2.id = 'd68214b2-e466-457f-a661-c8413a094a10'
        res = cluster.remove_node(node2)
        self.assertIsNone(res)
        self.assertEqual([node1], cluster.nodes)

        # add another node
        cluster.add_node(node2)
        self.assertEqual([node1, node2], cluster.nodes)

        # remove first node
        res = cluster.remove_node(node1.id)
        self.assertIsNone(res)
        self.assertEqual([node2], cluster.nodes)

        # reload and remove node
        node3 = mock.Mock()
        node3.id = 'd68214b2-e466-457f-a661-c8413a094a10'

        res = cluster.remove_node(node3.id)
        self.assertIsNone(res)
        self.assertEqual([], cluster.nodes)

    def test_update_node(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        self.assertEqual([], cluster.nodes)

        node1 = mock.Mock(id='fake', status='ACTIVE')
        # add one
        cluster.add_node(node1)

        node1.status = 'ERROR'
        cluster.update_node([node1])
        self.assertEqual([node1], cluster.nodes)

        # update new ones
        node2 = mock.Mock(id='fake1', status='ACTIVE')
        node3 = mock.Mock(id='fake2', status='ERROR')
        cluster.update_node([node2, node3])
        self.assertEqual([node2, node3], cluster.nodes)

    @mock.patch.object(pcb.Policy, 'load')
    @mock.patch.object(cpm, 'ClusterPolicy')
    def test_attach_policy(self, mock_cp, mock_load):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        cluster.id = CLUSTER_ID

        policy = mock.Mock()
        policy.attach.return_value = (True, None)
        policy.PRIORITY = 10
        mock_load.return_value = policy

        binding = mock.Mock()
        mock_cp.return_value = binding

        values = {'enabled': True}
        res, reason = cluster.attach_policy(self.context, POLICY_ID, values)
        policy.attach.assert_called_once_with(cluster, enabled=True)
        mock_load.assert_called_once_with(self.context, POLICY_ID)
        mock_cp.assert_called_once_with(CLUSTER_ID, POLICY_ID, priority=10,
                                        enabled=True, data=None)
        binding.store.assert_called_once_with(self.context)
        self.assertIn(policy, cluster.policies)

    @mock.patch.object(pcb.Policy, 'load')
    def test_attach_policy_already_attached(self, mock_load):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)

        policy_id = '62d52dd6-5f83-4340-b079-349da2f9ffd9'
        existing = mock.Mock(id=policy_id)
        cluster.rt['policies'] = [existing]
        policy = mock.Mock()
        mock_load.return_value = policy

        # do it
        res, reason = cluster.attach_policy(self.context, policy_id, {})

        self.assertTrue(res)
        self.assertEqual('Policy already attached.', reason)
        mock_load.assert_called_once_with(self.context, policy_id)

    @mock.patch.object(pcb.Policy, 'load')
    def test_attach_policy_type_conflict(self, mock_load):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        cluster.id = CLUSTER_ID

        existing = mock.Mock()
        existing.id = POLICY_ID
        existing.type = 'POLICY_TYPE_ONE'
        cluster.rt['policies'] = [existing]

        policy = mock.Mock()
        policy.singleton = True
        policy.type = 'POLICY_TYPE_ONE'
        mock_load.return_value = policy

        # do it
        new_policy_id = '62d52dd6-5f83-4340-b079-349da2f9ffd9'
        res, reason = cluster.attach_policy(self.context, new_policy_id, {})

        # assert
        self.assertFalse(res)
        expected = ('Only one instance of policy type (POLICY_TYPE_ONE) can '
                    'be attached to a cluster, but another instance '
                    '(%s) is found attached to the cluster '
                    '(%s) already.' % (POLICY_ID, CLUSTER_ID))
        self.assertEqual(expected, reason)
        mock_load.assert_called_once_with(self.context, new_policy_id)

    @mock.patch.object(cpm, 'ClusterPolicy')
    @mock.patch.object(pcb.Policy, 'load')
    def test_attach_policy_type_conflict_but_ok(self, mock_load, mock_cp):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)

        existing = mock.Mock()
        existing.id = POLICY_ID
        existing.type = 'POLICY_TYPE_ONE'
        cluster.rt['policies'] = [existing]

        policy = mock.Mock()
        policy.singleton = False
        policy.type = 'POLICY_TYPE_ONE'
        policy.attach.return_value = (True, None)
        policy.PRIORITY = 10
        mock_load.return_value = policy

        binding = mock.Mock()
        mock_cp.return_value = binding

        values = {'enabled': True}

        # do it
        new_policy_id = '62d52dd6-5f83-4340-b079-349da2f9ffd9'
        res, reason = cluster.attach_policy(self.context, new_policy_id,
                                            values)

        # assert
        self.assertTrue(res)
        self.assertEqual('Policy attached.', reason)

        policy.attach.assert_called_once_with(cluster, enabled=True)
        mock_load.assert_called_once_with(self.context, new_policy_id)
        mock_cp.assert_called_once_with(cluster.id, new_policy_id, priority=10,
                                        enabled=True, data=None)
        binding.store.assert_called_once_with(self.context)
        self.assertIn(policy, cluster.policies)

    @mock.patch.object(pcb.Policy, 'load')
    def test_attach_policy_failed_do_attach(self, mock_load):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)

        policy = mock.Mock()
        policy.attach.return_value = (False, 'Bad things happened.')
        mock_load.return_value = policy

        # do it
        new_id = '62d52dd6-5f83-4340-b079-349da2f9ffd9'
        res, reason = cluster.attach_policy(self.context, new_id, {})

        self.assertFalse(res)
        self.assertEqual('Bad things happened.', reason)
        policy.attach.assert_called_once_with(cluster, enabled=True)
        mock_load.assert_called_once_with(self.context, new_id)

    @mock.patch.object(cpo.ClusterPolicy, 'delete')
    @mock.patch.object(pcb.Policy, 'load')
    def test_detach_policy(self, mock_load, mock_detach):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        cluster.id = CLUSTER_ID

        policy = mock.Mock()
        policy.id = POLICY_ID
        existing = mock.Mock()
        existing.id = POLICY_ID
        cluster.rt['policies'] = [existing]
        policy.detach.return_value = (True, None)
        mock_load.return_value = policy

        res, reason = cluster.detach_policy(self.context, POLICY_ID)

        self.assertTrue(res)
        self.assertEqual('Policy detached.', reason)
        policy.detach.assert_called_once_with(cluster)
        mock_load.assert_called_once_with(self.context, POLICY_ID)
        mock_detach.assert_called_once_with(self.context, CLUSTER_ID,
                                            POLICY_ID)
        self.assertEqual([], cluster.rt['policies'])

    def test_detach_policy_not_attached(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        cluster.rt['policies'] = []

        res, reason = cluster.detach_policy(self.context, POLICY_ID)

        self.assertFalse(res)
        self.assertEqual('Policy not attached.', reason)

    @mock.patch.object(pcb.Policy, 'load')
    def test_detach_policy_failed_detach(self, mock_load):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        policy = mock.Mock()
        policy.id = POLICY_ID
        policy.detach.return_value = False, 'Things went wrong.'
        mock_load.return_value = policy
        cluster.rt['policies'] = [policy]

        res, reason = cluster.detach_policy(self.context, POLICY_ID)

        self.assertFalse(res)
        self.assertEqual('Things went wrong.', reason)
        mock_load.assert_called_once_with(self.context, POLICY_ID)
        policy.detach.assert_called_once_with(cluster)

    @mock.patch.object(cpo.ClusterPolicy, 'update')
    def test_update_policy(self, mock_update):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        cluster.id = CLUSTER_ID

        existing = mock.Mock()
        existing.id = POLICY_ID
        existing.type = "senlin.policy.foo"
        cluster.rt['policies'] = [existing]
        values = {
            'enabled': False
        }

        res, reason = cluster.update_policy(self.context, POLICY_ID, **values)

        self.assertTrue(res)
        self.assertEqual('Policy updated.', reason)
        mock_update.assert_called_once_with(
            self.context, CLUSTER_ID, POLICY_ID, {'enabled': False})

    def test_update_policy_not_attached(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        cluster.rt['policies'] = []
        values = {'enabled': False}

        # do it
        res, reason = cluster.update_policy(self.context, POLICY_ID, **values)

        self.assertFalse(res)
        self.assertEqual('Policy not attached.', reason)

    def test_update_policy_no_update_needed(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        existing = mock.Mock()
        existing.id = POLICY_ID
        cluster.rt['policies'] = [existing]
        values = {}

        # do it
        res, reason = cluster.update_policy(self.context, POLICY_ID, **values)

        self.assertTrue(res)
        self.assertEqual('No update is needed.', reason)

    @mock.patch.object(cpo.ClusterPolicy, "update")
    @mock.patch.object(health_manager, "enable")
    def test_update_policy_enable_health(self, mock_enable, mock_update):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID, id=CLUSTER_ID)
        existing = mock.Mock(id=POLICY_ID, type="senlin.policy.health")
        cluster.rt['policies'] = [existing]
        values = {"enabled": True}

        # do it
        res, reason = cluster.update_policy(self.context, POLICY_ID, **values)

        self.assertTrue(res)
        mock_enable.assert_called_once_with(CLUSTER_ID)
        mock_update.assert_called_once_with(
            self.context, CLUSTER_ID, POLICY_ID, {'enabled': True})

    @mock.patch.object(cpo.ClusterPolicy, "update")
    @mock.patch.object(health_manager, "disable")
    def test_update_policy_disable_health(self, mock_disable, mock_update):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID, id=CLUSTER_ID)
        existing = mock.Mock(id=POLICY_ID, type="senlin.policy.health")
        cluster.rt['policies'] = [existing]
        values = {"enabled": False}

        # do it
        res, reason = cluster.update_policy(self.context, POLICY_ID, **values)

        self.assertTrue(res)
        mock_disable.assert_called_once_with(CLUSTER_ID)
        mock_update.assert_called_once_with(
            self.context, CLUSTER_ID, POLICY_ID, {'enabled': False})

    def test_get_region_distribution(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)

        node1 = mock.Mock()
        node1.data = {'placement': {'region_name': 'R1'}}
        node2 = mock.Mock()
        node2.data = {'placement': {'region_name': 'R2'}}
        node3 = mock.Mock()
        node3.data = {'key': 'value'}

        node4 = mock.Mock()
        node4.data = {'placement': {'region_name': 'BAD'}}

        nodes = [node1, node2, node3, node4]
        for n in nodes:
            cluster.add_node(n)

        result = cluster.get_region_distribution(['R1', 'R2', 'R3'])

        self.assertEqual(3, len(result))
        self.assertEqual(1, result['R1'])
        self.assertEqual(1, result['R2'])
        self.assertEqual(0, result['R3'])

    def test_get_zone_distribution(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        node1 = mock.Mock()
        node1.data = {}
        node1.get_details.return_value = {
            'OS-EXT-AZ:availability_zone': 'AZ1',
        }
        node2 = mock.Mock()
        node2.data = {
            'foobar': 'irrelevant'
        }
        node3 = mock.Mock()
        node3.data = {
            'placement': {
                'zone': 'AZ2'
            }
        }

        nodes = [node1, node2, node3]
        for n in nodes:
            cluster.add_node(n)

        result = cluster.get_zone_distribution(self.context,
                                               ['AZ1', 'AZ2', 'AZ3'])

        self.assertEqual(3, len(result))
        self.assertEqual(1, result['AZ1'])
        self.assertEqual(1, result['AZ2'])
        self.assertEqual(0, result['AZ3'])

        node1.get_details.assert_called_once_with(self.context)

    def test_nodes_by_region(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        node1 = mock.Mock(data={'placement': {'region_name': 'R1'}})
        node2 = mock.Mock(data={'placement': {'region_name': 'R2'}})
        node3 = mock.Mock(data={'key': 'value'})
        node4 = mock.Mock(data={'placement': {'region_name': 'BAD'}})

        nodes = [node1, node2, node3, node4]
        for n in nodes:
            cluster.add_node(n)

        result = cluster.nodes_by_region('R1')
        self.assertEqual(1, len(result))
        self.assertEqual(node1, result[0])

        result = cluster.nodes_by_region('R2')
        self.assertEqual(1, len(result))
        self.assertEqual(node2, result[0])

        result = cluster.nodes_by_region('R3')
        self.assertEqual(0, len(result))

    def test_nodes_by_zone(self):
        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
        node1 = mock.Mock(data={'placement': {'zone': 'AZ1'}})
        node2 = mock.Mock(data={'placement': {'zone': 'AZ2'}})
        node3 = mock.Mock(data={'key': 'value'})
        node4 = mock.Mock(data={'placement': {'zone': 'BAD'}})

        nodes = [node1, node2, node3, node4]
        for n in nodes:
            cluster.add_node(n)

        result = cluster.nodes_by_zone('AZ1')
        self.assertEqual(1, len(result))
        self.assertEqual(node1, result[0])

        result = cluster.nodes_by_zone('AZ2')
        self.assertEqual(1, len(result))
        self.assertEqual(node2, result[0])

        result = cluster.nodes_by_region('AZ3')
        self.assertEqual(0, len(result))

    @mock.patch.object(node_mod.Node, 'load_all')
    @mock.patch.object(node_mod.Node, 'do_check')
    @mock.patch.object(cm.Cluster, 'update_node')
    def test_health_check(self, mock_update, mock_check, mock_load):
        cluster = cm.Cluster('test-cluster', 5, PROFILE_ID,
                             min_size=2, id=CLUSTER_ID)
        node1 = node_mod.Node('fake1', PROFILE_ID, status='ACTIVE')
        node2 = node_mod.Node('fake2', PROFILE_ID, status='ACTIVE')
        nodes = [node1, node2]
        for node in nodes:
            cluster.add_node(node)

        node1.status = 'ERROR'
        mock_load.return_value = [node1, node2]

        cluster.health_check(self.context)

        self.assertEqual(2, len(cluster.nodes))
        self.assertEqual([node1, node2], cluster.nodes)

        mock_update.assert_called_once_with([node1, node2])
        mock_check.assert_has_calls([
            mock.call(self.context),
            mock.call(self.context)
        ])
        mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)

    @mock.patch.object(co.Cluster, 'update')
    @mock.patch.object(node_mod.Node, 'load_all')
    def test_eval_status_below_min_size(self, mock_load, mock_update):
        cluster = cm.Cluster('test-cluster', 5, PROFILE_ID,
                             min_size=2, id=CLUSTER_ID)
        node1 = mock.Mock(status='ACTIVE')
        node2 = mock.Mock(status='ERROR')
        node3 = mock.Mock(status='WARNING')
        mock_load.return_value = [node1, node2, node3]

        cluster.eval_status(self.context, 'TEST')
        rt = cluster.rt
        self.assertEqual(3, len(rt['nodes']))
        self.assertIsInstance(rt['nodes'], list)
        mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)
        mock_update.assert_called_once_with(
            self.context, CLUSTER_ID,
            {'status': consts.CS_ERROR,
             'status_reason': 'TEST: number of active nodes is below '
                              'min_size (2).'})

    @mock.patch.object(co.Cluster, 'update')
    @mock.patch.object(node_mod.Node, 'load_all')
    def test_eval_status_below_desired_capacity(self, mock_load, mock_update):
        cluster = cm.Cluster('test-cluster', 5, PROFILE_ID,
                             min_size=1, id=CLUSTER_ID)
        node1 = mock.Mock(status='ACTIVE')
        node2 = mock.Mock(status='ERROR')
        node3 = mock.Mock(status='WARNING')
        mock_load.return_value = [node1, node2, node3]

        cluster.eval_status(self.context, 'TEST')

        mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)
        mock_update.assert_called_once_with(
            self.context, CLUSTER_ID,
            {'status': consts.CS_WARNING,
             'status_reason': 'TEST: number of active nodes is below '
                              'desired_capacity (5).'})

    @mock.patch.object(co.Cluster, 'update')
    @mock.patch.object(node_mod.Node, 'load_all')
    def test_eval_status_equal_desired_capacity(self, mock_load, mock_update):
        cluster = cm.Cluster('test-cluster', 3, PROFILE_ID,
                             min_size=1, id=CLUSTER_ID)
        node1 = mock.Mock(status='ACTIVE')
        node2 = mock.Mock(status='ACTIVE')
        node3 = mock.Mock(status='ACTIVE')
        mock_load.return_value = [node1, node2, node3]

        cluster.eval_status(self.context, 'TEST')

        mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)
        mock_update.assert_called_once_with(
            self.context, CLUSTER_ID,
            {'status': consts.CS_ACTIVE,
             'status_reason': 'TEST: number of active nodes is equal or above '
                              'desired_capacity (3).'})

    @mock.patch.object(co.Cluster, 'update')
    @mock.patch.object(node_mod.Node, 'load_all')
    def test_eval_status_above_desired_capacity(self, mock_load, mock_update):
        cluster = cm.Cluster('test-cluster', 2, PROFILE_ID,
                             min_size=1, id=CLUSTER_ID)
        node1 = mock.Mock(status='ACTIVE')
        node2 = mock.Mock(status='ACTIVE')
        node3 = mock.Mock(status='ACTIVE')
        mock_load.return_value = [node1, node2, node3]

        cluster.eval_status(self.context, 'TEST')

        mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)
        mock_update.assert_called_once_with(
            self.context, CLUSTER_ID,
            {'status': consts.CS_ACTIVE,
             'status_reason': 'TEST: number of active nodes is equal or above '
                              'desired_capacity (2).'})

    @mock.patch.object(co.Cluster, 'update')
    @mock.patch.object(node_mod.Node, 'load_all')
    def test_eval_status_above_max_size(self, mock_load, mock_update):
        cluster = cm.Cluster('test-cluster', 2, PROFILE_ID,
                             max_size=2, id=CLUSTER_ID)
        node1 = mock.Mock(status='ACTIVE')
        node2 = mock.Mock(status='ACTIVE')
        node3 = mock.Mock(status='ACTIVE')
        mock_load.return_value = [node1, node2, node3]

        cluster.eval_status(self.context, 'TEST')

        mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)
        mock_update.assert_called_once_with(
            self.context, CLUSTER_ID,
            {'status': consts.CS_WARNING,
             'status_reason': 'TEST: number of active nodes is above '
                              'max_size (2).'})

    @mock.patch.object(co.Cluster, 'update')
    @mock.patch.object(node_mod.Node, 'load_all')
    def test_eval_status_with_new_desired(self, mock_load, mock_update):
        cluster = cm.Cluster('test-cluster', 5, PROFILE_ID, id=CLUSTER_ID)
        node1 = mock.Mock(status='ACTIVE')
        node2 = mock.Mock(status='ERROR')
        node3 = mock.Mock(status='WARNING')
        mock_load.return_value = [node1, node2, node3]

        cluster.eval_status(self.context, 'TEST', desired_capacity=2)

        mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)
        mock_update.assert_called_once_with(
            self.context, CLUSTER_ID,
            {'desired_capacity': 2,
             'status': consts.CS_WARNING,
             'status_reason': 'TEST: number of active nodes is below '
                              'desired_capacity (2).'})

    @mock.patch.object(co.Cluster, 'update')
    @mock.patch.object(node_mod.Node, 'load_all')
    def test_eval_status__new_desired_is_zero(self, mock_load, mock_update):
        cluster = cm.Cluster('test-cluster', 5, PROFILE_ID, id=CLUSTER_ID)
        node1 = mock.Mock(status='ACTIVE')
        node2 = mock.Mock(status='ERROR')
        node3 = mock.Mock(status='WARNING')
        mock_load.return_value = [node1, node2, node3]

        cluster.eval_status(self.context, 'TEST', desired_capacity=0)

        mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)
        mock_update.assert_called_once_with(
            self.context, CLUSTER_ID,
            {'desired_capacity': 0,
             'status': consts.CS_ACTIVE,
             'status_reason': 'TEST: number of active nodes is equal or above '
                              'desired_capacity (0).'})

    @mock.patch.object(co.Cluster, 'update')
    @mock.patch.object(node_mod.Node, 'load_all')
    def test_eval_status_with_new_min(self, mock_load, mock_update):
        cluster = cm.Cluster('test-cluster', 5, PROFILE_ID,
                             id=CLUSTER_ID)
        node1 = mock.Mock(status='ACTIVE')
        node2 = mock.Mock(status='ERROR')
        node3 = mock.Mock(status='WARNING')
        mock_load.return_value = [node1, node2, node3]

        cluster.eval_status(self.context, 'TEST', min_size=2)

        mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)
        mock_update.assert_called_once_with(
            self.context, CLUSTER_ID,
            {'min_size': 2,
             'status': consts.CS_ERROR,
             'status_reason': 'TEST: number of active nodes is below '
                              'min_size (2).'})

    @mock.patch.object(co.Cluster, 'update')
    @mock.patch.object(node_mod.Node, 'load_all')
    def test_eval_status_with_new_max(self, mock_load, mock_update):
        cluster = cm.Cluster('test-cluster', 2, PROFILE_ID,
                             max_size=5, id=CLUSTER_ID)
        node1 = mock.Mock(status='ACTIVE')
        node2 = mock.Mock(status='ACTIVE')
        node3 = mock.Mock(status='ACTIVE')
        mock_load.return_value = [node1, node2, node3]

        cluster.eval_status(self.context, 'TEST', max_size=6)

        mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)
        mock_update.assert_called_once_with(
            self.context, CLUSTER_ID,
            {'max_size': 6,
             'status': consts.CS_ACTIVE,
             'status_reason': 'TEST: number of active nodes is equal or above '
                              'desired_capacity (2).'})



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import six

from senlin.api.middleware import trust
from senlin.common import context
from senlin.common import exception
from senlin.objects.requests import credentials as vorc
from senlin.tests.unit.common import base
from senlin.tests.unit.common import utils


class TestTrustMiddleware(base.SenlinTestCase):

    def setUp(self):
        super(TestTrustMiddleware, self).setUp()
        self.context = utils.dummy_context()
        self.req = mock.Mock
        self.req.context = self.context
        self.middleware = trust.TrustMiddleware(None)

    @mock.patch("senlin.rpc.client.EngineClient")
    def test__get_trust_already_exists(self, mock_rpc):
        x_cred = {'trust': 'FAKE_TRUST_ID'}
        x_rpc = mock.Mock()
        x_rpc.call.return_value = x_cred
        mock_rpc.return_value = x_rpc

        result = self.middleware._get_trust(self.req)

        self.assertEqual('FAKE_TRUST_ID', result)
        mock_rpc.assert_called_once_with()
        x_rpc.call.assert_called_once_with(self.context, 'credential_get',
                                           mock.ANY)
        request = x_rpc.call.call_args[0][2]
        self.assertIsInstance(request, vorc.CredentialGetRequest)
        self.assertEqual(self.context.user, request.user)
        self.assertEqual(self.context.project, request.project)

    @mock.patch.object(context, "get_service_credentials")
    @mock.patch("senlin.drivers.base.SenlinDriver")
    @mock.patch("senlin.rpc.client.EngineClient")
    def test__get_trust_bad(self, mock_rpc, mock_driver, mock_creds):
        x_cred = {'foo': 'bar'}
        x_rpc = mock.Mock()
        x_rpc.call.return_value = x_cred
        mock_rpc.return_value = x_rpc

        x_svc_cred = {'uid': 'FAKE_ID', 'passwd': 'FAKE_PASS'}
        mock_creds.return_value = x_svc_cred
        x_admin_id = 'FAKE_ADMIN_ID'
        x_trust = mock.Mock(id='FAKE_TRUST_ID')
        mock_keystone = mock.Mock()
        mock_keystone.get_user_id.return_value = x_admin_id
        mock_keystone.trust_get_by_trustor.return_value = x_trust
        x_driver = mock.Mock()
        x_driver.identity.return_value = mock_keystone
        mock_driver.return_value = x_driver

        result = self.middleware._get_trust(self.req)
        self.assertEqual('FAKE_TRUST_ID', result)
        mock_calls = [mock.call(self.context, 'credential_get', mock.ANY),
                      mock.call(self.context, 'credential_create', mock.ANY)]
        x_rpc.call.assert_has_calls(mock_calls)
        request = x_rpc.call.call_args_list[0][0][2]
        self.assertIsInstance(request, vorc.CredentialGetRequest)
        self.assertEqual(self.context.user, request.user)
        self.assertEqual(self.context.project, request.project)
        request = x_rpc.call.call_args_list[1][0][2]
        self.assertIsInstance(request, vorc.CredentialCreateRequest)
        expected_cred = {
            'openstack': {'trust': 'FAKE_TRUST_ID'}
        }
        self.assertEqual(expected_cred, request.cred)
        mock_driver.assert_called_once_with()
        x_driver.identity.assert_called_once_with({
            'auth_url': self.context.auth_url,
            'project_id': self.context.project,
            'user_id': self.context.user,
            'token': self.context.auth_token,
        })
        mock_creds.assert_called_once_with()
        mock_keystone.get_user_id.assert_called_once_with(
            uid='FAKE_ID', passwd='FAKE_PASS')
        mock_keystone.trust_get_by_trustor.assert_called_once_with(
            self.context.user, 'FAKE_ADMIN_ID', self.context.project)

    @mock.patch.object(context, "get_service_credentials")
    @mock.patch("senlin.drivers.base.SenlinDriver")
    @mock.patch("senlin.rpc.client.EngineClient")
    def test__get_trust_not_found(self, mock_rpc, mock_driver, mock_creds):
        x_rpc = mock.Mock()
        x_rpc.call.return_value = None
        mock_rpc.return_value = x_rpc

        x_svc_cred = {'uid': 'FAKE_ID', 'passwd': 'FAKE_PASS'}
        mock_creds.return_value = x_svc_cred
        x_admin_id = 'FAKE_ADMIN_ID'
        x_trust = mock.Mock(id='FAKE_TRUST_ID')
        mock_keystone = mock.Mock()
        mock_keystone.get_user_id.return_value = x_admin_id
        mock_keystone.trust_get_by_trustor.return_value = x_trust
        x_driver = mock.Mock()
        x_driver.identity.return_value = mock_keystone
        mock_driver.return_value = x_driver

        result = self.middleware._get_trust(self.req)
        self.assertEqual('FAKE_TRUST_ID', result)
        mock_calls = [mock.call(self.context, 'credential_get', mock.ANY),
                      mock.call(self.context, 'credential_create', mock.ANY)]
        x_rpc.call.assert_has_calls(mock_calls)
        mock_rpc.assert_called_once_with()
        mock_driver.assert_called_once_with()
        x_driver.identity.assert_called_once_with({
            'auth_url': self.context.auth_url,
            'project_id': self.context.project,
            'user_id': self.context.user,
            'token': self.context.auth_token,
        })
        mock_creds.assert_called_once_with()
        mock_keystone.get_user_id.assert_called_once_with(
            uid='FAKE_ID', passwd='FAKE_PASS')
        mock_keystone.trust_get_by_trustor.assert_called_once_with(
            self.context.user, 'FAKE_ADMIN_ID', self.context.project)

    @mock.patch.object(context, "get_service_credentials")
    @mock.patch("senlin.drivers.base.SenlinDriver")
    @mock.patch("senlin.rpc.client.EngineClient")
    def test__get_trust_do_create(self, mock_rpc, mock_driver, mock_creds):
        x_rpc = mock.Mock()
        x_rpc.call.return_value = None
        mock_rpc.return_value = x_rpc

        x_svc_cred = {'uid': 'FAKE_ID', 'passwd': 'FAKE_PASS'}
        mock_creds.return_value = x_svc_cred
        x_admin_id = 'FAKE_ADMIN_ID'
        mock_keystone = mock.Mock()
        mock_keystone.get_user_id.return_value = x_admin_id
        x_trust = mock.Mock(id='FAKE_TRUST_ID')
        mock_keystone.trust_create.return_value = x_trust
        err = exception.InternalError(code=400, message='Boom')
        mock_keystone.trust_get_by_trustor.side_effect = err
        x_driver = mock.Mock()
        x_driver.identity.return_value = mock_keystone
        mock_driver.return_value = x_driver

        result = self.middleware._get_trust(self.req)
        self.assertEqual('FAKE_TRUST_ID', result)
        mock_calls = [mock.call(self.context, 'credential_get', mock.ANY),
                      mock.call(self.context, 'credential_create', mock.ANY)]
        x_rpc.call.assert_has_calls(mock_calls)
        mock_driver.assert_called_once_with()
        x_driver.identity.assert_called_once_with({
            'auth_url': self.context.auth_url,
            'project_id': self.context.project,
            'user_id': self.context.user,
            'token': self.context.auth_token,
        })
        mock_creds.assert_called_once_with()
        mock_keystone.get_user_id.assert_called_once_with(
            uid='FAKE_ID', passwd='FAKE_PASS')
        mock_keystone.trust_get_by_trustor.assert_called_once_with(
            self.context.user, 'FAKE_ADMIN_ID', self.context.project)
        mock_keystone.trust_create.assert_called_once_with(
            self.context.user, 'FAKE_ADMIN_ID', self.context.project,
            self.context.roles)

    @mock.patch.object(context, "get_service_credentials")
    @mock.patch("senlin.drivers.base.SenlinDriver")
    @mock.patch("senlin.rpc.client.EngineClient")
    def test__get_trust_fatal(self, mock_rpc, mock_driver, mock_creds):
        x_rpc = mock.Mock()
        x_rpc.call.return_value = None
        mock_rpc.return_value = x_rpc

        x_svc_cred = {'uid': 'FAKE_ID', 'passwd': 'FAKE_PASS'}
        mock_creds.return_value = x_svc_cred
        x_admin_id = 'FAKE_ADMIN_ID'
        mock_keystone = mock.Mock()
        mock_keystone.get_user_id.return_value = x_admin_id
        err = exception.InternalError(code=500, message='Boom')
        mock_keystone.trust_get_by_trustor.side_effect = err
        x_driver = mock.Mock()
        x_driver.identity.return_value = mock_keystone
        mock_driver.return_value = x_driver

        ex = self.assertRaises(exception.InternalError,
                               self.middleware._get_trust,
                               self.req)

        self.assertEqual('Boom', six.text_type(ex))
        mock_rpc.assert_called_once_with()
        x_rpc.call.assert_called_once_with(self.context, 'credential_get',
                                           mock.ANY)
        mock_driver.assert_called_once_with()
        x_driver.identity.assert_called_once_with({
            'auth_url': self.context.auth_url,
            'project_id': self.context.project,
            'user_id': self.context.user,
            'token': self.context.auth_token,
        })
        mock_creds.assert_called_once_with()
        mock_keystone.get_user_id.assert_called_once_with(
            uid='FAKE_ID', passwd='FAKE_PASS')
        mock_keystone.trust_get_by_trustor.assert_called_once_with(
            self.context.user, 'FAKE_ADMIN_ID', self.context.project)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from senlin.tests.tempest.api import base
from senlin.tests.tempest.common import utils


class TestPolicyUpdate(base.BaseSenlinAPITest):

    def setUp(self):
        super(TestPolicyUpdate, self).setUp()
        self.policy_id = utils.create_a_policy(self)
        self.addCleanup(utils.delete_a_policy, self, self.policy_id)

    @decorators.idempotent_id('dbe9c6a6-882c-41cf-b862-7f648804db24')
    def test_policy_update(self):
        params = {
            'policy': {
                'name': 'updated-policy-name'
            }
        }
        res = self.client.update_obj('policies', self.policy_id, params)

        # Verify resp of policy update API
        self.assertEqual(200, res['status'])
        self.assertIsNotNone(res['body'])
        policy = res['body']
        for key in ['created_at', 'data', 'domain', 'id', 'name', 'project',
                    'spec', 'type', 'updated_at', 'user']:
            self.assertIn(key, policy)
        self.assertEqual('updated-policy-name', policy['name'])



#!/usr/bin/env python
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""WSGI script for senlin-api.

Use this file for deploying senlin-api under Apache2(mode-wsgi).
"""


from oslo_config import cfg
import oslo_i18n as i18n
from oslo_log import log as logging

from senlin.api.common import wsgi
from senlin.common import config
from senlin.common import messaging
from senlin.common import profiler
from senlin import version


def init_app():
    i18n.enable_lazy()

    logging.register_options(cfg.CONF)
    cfg.CONF(project='senlin', prog='senlin-api',
             version=version.version_info.version_string())
    logging.setup(cfg.CONF, 'senlin-api')
    config.set_config_defaults()
    messaging.setup()

    profiler.setup('senlin-api', cfg.CONF.host)
    return wsgi.load_paste_app()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
from oslo_config import cfg
from oslo_versionedobjects import fields
import six
import testtools

from senlin.common import consts
from senlin.objects import base
from senlin.objects import fields as senlin_fields

CONF = cfg.CONF


class FakeFieldType(fields.FieldType):
    def coerce(self, obj, attr, value):
        return '*%s*' % value

    def to_primitive(self, obj, attr, value):
        return '!%s!' % value

    def from_primitive(self, obj, attr, value):
        return value[1:-1]


class TestField(testtools.TestCase):

    def setUp(self):
        super(TestField, self).setUp()
        self.field = fields.Field(FakeFieldType())
        self.coerce_good_values = [('foo', '*foo*')]
        self.coerce_bad_values = []
        self.to_primitive_values = [('foo', '!foo!')]
        self.from_primitive_values = [('!foo!', 'foo')]

    def test_coerce_good_values(self):
        for in_val, out_val in self.coerce_good_values:
            self.assertEqual(out_val, self.field.coerce('obj', 'attr', in_val))

    def test_coerce_bad_values(self):
        for in_val in self.coerce_bad_values:
            self.assertRaises((TypeError, ValueError),
                              self.field.coerce, 'obj', 'attr', in_val)

    def test_to_primitive(self):
        for in_val, prim_val in self.to_primitive_values:
            self.assertEqual(prim_val,
                             self.field.to_primitive('obj', 'attr', in_val))

    def test_from_primitive(self):
        class ObjectLikeThing(object):
            _context = 'context'

        for prim_val, out_val in self.from_primitive_values:
            self.assertEqual(out_val,
                             self.field.from_primitive(ObjectLikeThing, 'attr',
                                                       prim_val))

    def test_stringify(self):
        self.assertEqual('123', self.field.stringify(123))


class TestBoolean(TestField):

    def setUp(self):
        super(TestBoolean, self).setUp()

        self.field = senlin_fields.BooleanField()
        self.coerce_good_values = [
            ('True', True),
            ('T', True),
            ('t', True),
            ('1', True),
            ('yes', True),
            ('on', True),
            ('False', False),
            ('F', False),
            ('f', False),
            ('0', False),
            ('no', False),
            ('off', False)
        ]
        self.coerce_bad_values = ['BOGUS']

        self.to_primitive_values = [
            (True, True),
            (False, False)
        ]

        self.from_primitive_values = [
            ('True', 'True'),
            ('False', 'False')
        ]

    def test_stringify(self):
        self.assertEqual('True', self.field.stringify(True))
        self.assertEqual('False', self.field.stringify(False))


class TestObject(TestField):

    def setUp(self):
        super(TestObject, self).setUp()

        @base.base.VersionedObjectRegistry.register
        class TestableObject(base.base.VersionedObject):
            fields = {
                'uuid': fields.StringField(),
            }

        test_inst = TestableObject()
        self._test_cls = TestableObject
        self.field = fields.Field(senlin_fields.Object('TestableObject'))
        self.coerce_good_values = [(test_inst, test_inst)]
        self.coerce_bad_values = [1, 'foo']
        self.to_primitive_values = [(test_inst, test_inst.obj_to_primitive())]
        self.from_primitive_values = [(test_inst.obj_to_primitive(),
                                       test_inst),
                                      (test_inst, test_inst)]

    def test_from_primitive(self):
        # we ignore this test case
        pass

    def test_stringify(self):
        # and this one is ignored as well
        pass

    def test_get_schema(self):
        self.assertEqual(
            {
                'properties': {
                    'versioned_object.changes': {
                        'items': {'type': 'string'}, 'type': 'array'
                    },
                    'versioned_object.data': {
                        'additionalProperties': False,
                        'description': 'fields of TestableObject',
                        'properties': {
                            'uuid': {'readonly': False, 'type': ['string']}
                        },
                        'required': ['uuid'],
                        'type': 'object',
                    },
                    'versioned_object.name': {
                        'type': 'string',
                    },
                    'versioned_object.namespace': {
                        'type': 'string',
                    },
                    'versioned_object.version': {
                        'type': 'string',
                    },
                },
                'readonly': False,
                'required': [
                    'versioned_object.namespace',
                    'versioned_object.name',
                    'versioned_object.version',
                    'versioned_object.data',
                ],
                'type': 'object',
            },
            self.field.get_schema())


class TestDummyObject(testtools.TestCase):

    def setUp(self):
        super(TestDummyObject, self).setUp()

        @base.base.VersionedObjectRegistry.register
        class TestableObject(base.base.VersionedObject):
            fields = {}

        self.field = fields.Field(senlin_fields.Object('TestableObject'))

    def test_get_schema(self):
        self.assertEqual(
            {
                'properties': {
                    'versioned_object.changes': {
                        'items': {'type': 'string'}, 'type': 'array'
                    },
                    'versioned_object.data': {
                        'additionalProperties': False,
                        'description': 'fields of TestableObject',
                        'properties': {},
                        'type': 'object',
                    },
                    'versioned_object.name': {
                        'type': 'string',
                    },
                    'versioned_object.namespace': {
                        'type': 'string',
                    },
                    'versioned_object.version': {
                        'type': 'string',
                    },
                },
                'readonly': False,
                'required': [
                    'versioned_object.namespace',
                    'versioned_object.name',
                    'versioned_object.version',
                    'versioned_object.data',
                ],
                'type': 'object',
            },
            self.field.get_schema())


class TestJson(TestField):
    def setUp(self):
        super(TestJson, self).setUp()

        self.field = senlin_fields.JsonField()
        self.coerce_good_values = [('{"k": "v"}', {"k": "v"})]
        self.coerce_bad_values = ['{"K": "v"]']
        self.to_primitive_values = [({"k": "v"}, '{"k": "v"}')]
        self.from_primitive_values = [('{"k": "v"}', {"k": "v"})]

    def test_stringify(self):
        self.assertEqual("{'k': 'v'}", self.field.stringify({"k": "v"}))

    def test_stringify_invalid(self):
        self.assertRaises(ValueError,
                          self.field.stringify, self.coerce_bad_values[0])

    def test_get_schema(self):
        self.assertEqual(
            {'type': ['object'], 'readonly': False},
            self.field.get_schema()
        )


class TestUniqueDict(TestField):

    def setUp(self):
        super(TestUniqueDict, self).setUp()

        self.field = senlin_fields.UniqueDict(fields.String())
        self.coerce_good_values = [({"k": "v"}, {"k": "v"})]
        self.coerce_bad_values = ['{"K": "v"]']
        self.to_primitive_values = [({"k": "v"}, {"k": "v"})]
        self.from_primitive_values = [({"k": "v"}, {"k": "v"})]

    def test_stringify(self):
        self.assertEqual("{k='v'}", self.field.stringify({"k": "v"}))

    def test_coerce(self):
        res = self.field.coerce(None, 'attr', {'k1': 'v1'})
        self.assertEqual({'k1': 'v1'}, res)

    def test_coerce_failed_duplicate(self):
        ex = self.assertRaises(ValueError,
                               self.field.coerce,
                               None, 'attr', {'k1': 'v1', 'k2': 'v1'})

        self.assertEqual('Map contains duplicated values',
                         six.text_type(ex))


class TestNotificationPriority(TestField):
    def setUp(self):
        super(TestNotificationPriority, self).setUp()

        self.field = senlin_fields.NotificationPriorityField()
        self.coerce_good_values = [('audit', 'audit'),
                                   ('critical', 'critical'),
                                   ('debug', 'debug'),
                                   ('error', 'error'),
                                   ('sample', 'sample'),
                                   ('warn', 'warn')]
        self.coerce_bad_values = ['warning']
        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual("'warn'", self.field.stringify('warn'))

    def test_stringify_invalid(self):
        self.assertRaises(ValueError, self.field.stringify, 'warning')


class TestNotificationPhase(TestField):
    def setUp(self):
        super(TestNotificationPhase, self).setUp()

        self.field = senlin_fields.NotificationPhaseField()
        self.coerce_good_values = [('start', 'start'),
                                   ('end', 'end'),
                                   ('error', 'error')]
        self.coerce_bad_values = ['begin']

        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual("'error'", self.field.stringify('error'))

    def test_stringify_invalid(self):
        self.assertRaises(ValueError, self.field.stringify, 'begin')


class TestName(TestField):

    def setUp(self):
        super(TestName, self).setUp()

        self.field = senlin_fields.NameField()
        self.coerce_good_values = [
            ('name1', 'name1'),          # plain string
            ('name2.sec', 'name2.sec'),  # '.' okay
            ('123-sec', '123-sec'),      # '-' okay
            ('123_sec', '123_sec'),      # '_' okay
            ('123~sec', '123~sec'),      # '~' okay
            ('557', '557'),              # pure numeric okay
        ]
        self.coerce_bad_values = [
            '',              # too short
            's' * 300,       # too long
            'ab/',           # '/' illegal
            's123$',         # '$' illegal
            '13^gadf',       # '^' illegal
            'sad&cheer',     # '&' illegal
            'boo**',         # '*' illegal
            'kwsqu()',       # '(' and ')' illegal
            'bing+bang',     # '+' illegal
            'var=value',     # '=' illegal
            'quicksort[1]',  # '[' and ']' illegal
            'sdi{"gh"}',     # '{' and '}' illegal
            'gate open',     # ' ' illegal
            '12.64%',        # '%' illegal
            'name#sign',     # '#' illegal
            'back\slash',    # '\' illegal
            ' leading',      # leading blank illegal
            'trailing ',     # trailing blank illegal
            '!okay',         # '!' illegal
            '@author',       # '@' illegal
            '`info`',        # '`' illegal
            '"partial',      # '"' illegal
            "'single",       # ''' illegal
            '<max',          # '<' illegal
            '>min',          # '>' illegal
            'question?',     # '?' illegal
            'first,second',  # ',' illegal
        ]
        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual("'name1'", self.field.stringify('name1'))

    def test_init(self):
        sot = senlin_fields.Name(2, 200)

        self.assertEqual(2, sot.min_len)
        self.assertEqual(200, sot.max_len)

    def test_coerce_failed(self):
        obj = mock.Mock()
        sot = senlin_fields.Name()

        ex = self.assertRaises(ValueError,
                               sot.coerce,
                               obj, 'attr', 'value/bad')
        self.assertEqual("The value for the 'attr' (value/bad) contains "
                         "illegal characters. It must contain only "
                         "alphanumeric or \"_-.~\" characters and must start "
                         "with letter.",
                         six.text_type(ex))

    def test_get_schema(self):
        sot = senlin_fields.Name(2, 200)
        self.assertEqual(
            {
                'type': ['string'],
                'minLength': 2,
                'maxLength': 200
            },
            sot.get_schema()
        )

    def test_get_schema_default(self):
        sot = senlin_fields.Name()
        self.assertEqual(
            {
                'type': ['string'],
                'minLength': 1,
                'maxLength': 255
            },
            sot.get_schema()
        )


class TestCapacity(TestField):

    def setUp(self):
        super(TestCapacity, self).setUp()

        self.field = senlin_fields.CapacityField()
        self.coerce_good_values = [
            (100, 100),          # plain integer
            ('100', 100),        # string of integer
            ('0123', 123),       # leading zeros ignored
        ]
        self.coerce_bad_values = [
            -1,              # less than 0
            'strval',        # illegal value
        ]
        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual('100', self.field.stringify(100))
        self.assertEqual('100', self.field.stringify('100'))

    def test_init(self):
        CONF.set_override('max_nodes_per_cluster', 300)
        sot = senlin_fields.Capacity()

        self.assertEqual(0, sot.minimum)
        self.assertEqual(300, sot.maximum)

    def test_init_with_values(self):
        CONF.set_override('max_nodes_per_cluster', 300)
        sot = senlin_fields.Capacity(2, 200)

        self.assertEqual(2, sot.minimum)
        self.assertEqual(200, sot.maximum)

    def test_init_invalid(self):
        CONF.set_override('max_nodes_per_cluster', 100)

        ex = self.assertRaises(ValueError,
                               senlin_fields.Capacity,
                               minimum=101)
        self.assertEqual("The value of 'minimum' cannot be greater than the "
                         "global constraint (100).", six.text_type(ex))

        ex = self.assertRaises(ValueError,
                               senlin_fields.Capacity,
                               maximum=101)
        self.assertEqual("The value of 'maximum' cannot be greater than the "
                         "global constraint (100).", six.text_type(ex))

        ex = self.assertRaises(ValueError,
                               senlin_fields.Capacity,
                               minimum=60, maximum=40)
        self.assertEqual("The value of 'maximum' must be greater than or equal"
                         " to that of the 'minimum' specified.",
                         six.text_type(ex))

    def test_coerce(self):
        sot = senlin_fields.Capacity(minimum=2, maximum=200)
        obj = mock.Mock()
        res = sot.coerce(obj, 'attr', 12)
        self.assertEqual(12, res)
        res = sot.coerce(obj, 'attr', 2)
        self.assertEqual(2, res)
        res = sot.coerce(obj, 'attr', 200)
        self.assertEqual(200, res)

        sot = senlin_fields.Capacity()

        res = sot.coerce(obj, 'attr', 12)
        self.assertEqual(12, res)
        res = sot.coerce(obj, 'attr', 0)
        self.assertEqual(0, res)
        res = sot.coerce(obj, 'attr', CONF.max_nodes_per_cluster)
        self.assertEqual(CONF.max_nodes_per_cluster, res)

    def test_coerce_failed(self):
        sot = senlin_fields.Capacity(minimum=2, maximum=200)
        obj = mock.Mock()

        ex = self.assertRaises(ValueError,
                               sot.coerce,
                               obj, 'attr', 1)
        self.assertEqual("The value for the attr field must be greater than "
                         "or equal to 2.", six.text_type(ex))

        ex = self.assertRaises(ValueError,
                               sot.coerce,
                               obj, 'attr', 201)
        self.assertEqual("The value for the attr field must be less than "
                         "or equal to 200.", six.text_type(ex))

        ex = self.assertRaises(ValueError,
                               sot.coerce,
                               obj, 'attr', 'badvalue')
        self.assertEqual("The value for attr must be an integer: 'badvalue'.",
                         six.text_type(ex))

    def test_get_schema(self):
        sot = senlin_fields.Capacity(minimum=2, maximum=200)
        self.assertEqual(
            {
                'type': ['integer', 'string'],
                'minimum': 2,
                'maximum': 200,
                'pattern': '^[0-9]*$',
            },
            sot.get_schema()
        )

    def test_get_schema_default(self):
        cfg.CONF.set_override('max_nodes_per_cluster', 100)
        sot = senlin_fields.Capacity()
        self.assertEqual(
            {
                'type': ['integer', 'string'],
                'minimum': 0,
                'maximum': 100,
                'pattern': '^[0-9]*$',
            },
            sot.get_schema()
        )


class TestSort(TestField):

    def setUp(self):
        super(TestSort, self).setUp()

        self.keys = ['key1', 'key2', 'key3']
        self.field = senlin_fields.Sort(valid_keys=self.keys)
        self.coerce_good_values = [
            ('key1', 'key1'),               # single key
            ('key1,key2', 'key1,key2'),     # multi keys
            ('key1:asc', 'key1:asc'),       # key with dir
            ('key2:desc', 'key2:desc'),     # key with different dir
            ('key1,key2:asc', 'key1,key2:asc'),  # mixed case
        ]
        self.coerce_bad_values = [
            'foo',              # unknown key
            ':desc',            # unspecified key
            'key1:up',          # unsupported dir
            'key1,key2:up',     # unsupported dir
            'foo,key2',         # unknown key
            'key2,:asc',        # unspecified key
            'key2,:desc',       # unspecified key
            'key1,',            # missing key
            ',key2',            # missing key
        ]
        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual("'key1,key2'", self.field.stringify('key1,key2'))

    def test_init(self):
        keys = ['foo', 'bar']
        sot = senlin_fields.Sort(valid_keys=keys)

        self.assertEqual(keys, sot.valid_keys)

    def test_coerce_failure(self):
        obj = mock.Mock()
        ex = self.assertRaises(ValueError,
                               self.field.coerce,
                               obj, 'attr', ':asc')
        self.assertEqual("Missing sort key for 'attr'.", six.text_type(ex))

        ex = self.assertRaises(ValueError,
                               self.field.coerce,
                               obj, 'attr', 'foo:asc')
        self.assertEqual("Unsupported sort key 'foo' for 'attr'.",
                         six.text_type(ex))

        ex = self.assertRaises(ValueError,
                               self.field.coerce,
                               obj, 'attr', 'key1:down')
        self.assertEqual("Unsupported sort dir 'down' for 'attr'.",
                         six.text_type(ex))

    def test_get_schema(self):
        self.assertEqual(
            {'type': ['string']},
            self.field.get_schema()
        )


class TestIdentityList(TestField):

    def setUp(self):
        super(TestIdentityList, self).setUp()

        self.field = senlin_fields.IdentityList(fields.String())

        self.coerce_good_values = [
            (['abc'], ['abc'])
        ]
        self.coerce_bad_values = [
            123
        ]
        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual("['abc','def']",
                         self.field.stringify(['abc', 'def']))

    def test_init_with_params(self):
        sot = senlin_fields.IdentityList(fields.String(), min_items=1,
                                         unique=False)

        self.assertEqual(1, sot.min_items)
        self.assertFalse(sot.unique_items)

    def test_coerce_not_unique_okay(self):
        sot = senlin_fields.IdentityList(fields.String(), min_items=1,
                                         unique=False)
        obj = mock.Mock()

        # not unique is okay
        res = sot.coerce(obj, 'attr', ['abc', 'abc'])
        self.assertEqual(['abc', 'abc'], res)

    def test_coerce_too_short(self):
        sot = senlin_fields.IdentityList(fields.String(), min_items=2,
                                         unique=False)
        obj = mock.Mock()

        # violating min_items
        ex = self.assertRaises(ValueError,
                               sot.coerce,
                               obj, 'attr', [])

        self.assertEqual("Value for 'attr' must have at least 2 item(s).",
                         six.text_type(ex))

    def test_coerce_not_unique_bad(self):
        obj = mock.Mock()

        # violating min_items
        ex = self.assertRaises(ValueError,
                               self.field.coerce,
                               obj, 'attr', ['abc', 'abc'])

        self.assertEqual("Items for 'attr' must be unique",
                         six.text_type(ex))

    def test_get_schema(self):
        self.assertEqual(
            {
                'type': ['array'],
                'items': {
                    'readonly': False,
                    'type': ['string'],
                },
                'minItems': 0,
                'uniqueItems': True
            },
            self.field.get_schema()
        )

        sot = senlin_fields.IdentityList(fields.String(), min_items=2,
                                         unique=False, nullable=True)
        self.assertEqual(
            {
                'type': ['array', 'null'],
                'items': {
                    'readonly': False,
                    'type': ['string'],
                },
                'minItems': 2,
                'uniqueItems': False
            },
            sot.get_schema()
        )


class TestAdjustmentTypeField(TestField):

    def setUp(self):
        super(TestAdjustmentTypeField, self).setUp()

        self.field = senlin_fields.AdjustmentTypeField()
        self.coerce_good_values = [
            ('EXACT_CAPACITY', 'EXACT_CAPACITY'),
            ('CHANGE_IN_CAPACITY', 'CHANGE_IN_CAPACITY'),
            ('CHANGE_IN_PERCENTAGE', 'CHANGE_IN_PERCENTAGE')
        ]
        self.coerce_bad_values = ['BOGUS']

        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual("'EXACT_CAPACITY'",
                         self.field.stringify('EXACT_CAPACITY'))

    def test_get_schema(self):
        self.assertEqual(
            {
                'type': ['string'],
                'readonly': False,
                'enum': ['EXACT_CAPACITY', 'CHANGE_IN_CAPACITY',
                         'CHANGE_IN_PERCENTAGE']
            },
            self.field.get_schema()
        )


class TestAdjustmentType(TestField):
    def setUp(self):
        super(TestAdjustmentType, self).setUp()

        self.field = senlin_fields.AdjustmentType()
        self.coerce_good_values = [
            ('EXACT_CAPACITY', 'EXACT_CAPACITY'),
            ('CHANGE_IN_CAPACITY', 'CHANGE_IN_CAPACITY'),
            ('CHANGE_IN_PERCENTAGE', 'CHANGE_IN_PERCENTAGE')
        ]
        self.coerce_bad_values = ['BOGUS']

        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual("'EXACT_CAPACITY'",
                         self.field.stringify('EXACT_CAPACITY'))

    def test_get_schema(self):
        self.assertEqual(
            {
                'type': ['string'],
                'enum': ['EXACT_CAPACITY', 'CHANGE_IN_CAPACITY',
                         'CHANGE_IN_PERCENTAGE']
            },
            self.field.get_schema()
        )


class TestClusterActionName(TestField):

    def setUp(self):
        super(TestClusterActionName, self).setUp()
        self.field = senlin_fields.ClusterActionNameField()
        self.coerce_good_values = [
            (action, action) for action in consts.CLUSTER_ACTION_NAMES]
        self.coerce_bad_values = ['BOGUS']

        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual("'CLUSTER_RESIZE'",
                         self.field.stringify('CLUSTER_RESIZE'))


class TestReceiverType(TestField):

    def setUp(self):
        super(TestReceiverType, self).setUp()
        self.field = senlin_fields.ReceiverTypeField()
        self.coerce_good_values = [
            (action, action) for action in consts.RECEIVER_TYPES]
        self.coerce_bad_values = ['BOGUS']

        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual("'message'",
                         self.field.stringify('message'))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock

from senlin.drivers.openstack import ceilometer_v2
from senlin.drivers.openstack import sdk
from senlin.tests.unit.common import base
from senlin.tests.unit.common import utils


class TestCeilometerV2(base.SenlinTestCase):

    def setUp(self):
        super(TestCeilometerV2, self).setUp()

        self.ctx = utils.dummy_context()
        self.conn_params = self.ctx.to_dict()
        self.mock_conn = mock.Mock()
        self.mock_create = self.patchobject(
            sdk, 'create_connection',
            return_value=self.mock_conn)

    def test_init(self):
        d = ceilometer_v2.CeilometerClient(self.conn_params)
        self.mock_create.assert_called_once_with(self.conn_params)
        self.assertEqual(self.mock_conn, d.conn)

    def test_alarm_create(self):
        d = ceilometer_v2.CeilometerClient(self.conn_params)
        d.alarm_create(name='test_alarm')
        self.mock_conn.telemetry.create_alarm.assert_called_once_with(
            name='test_alarm')

    def test_alarm_delete(self):
        d = ceilometer_v2.CeilometerClient(self.conn_params)
        d.alarm_delete('foo', True)
        self.mock_conn.telemetry.delete_alarm.assert_called_once_with(
            'foo', True)

        self.mock_conn.telemetry.delete_alarm.reset_mock()

        d.alarm_delete('foo', False)
        self.mock_conn.telemetry.delete_alarm.assert_called_once_with(
            'foo', False)

        self.mock_conn.telemetry.delete_alarm.reset_mock()

        d.alarm_delete('foo')
        self.mock_conn.telemetry.delete_alarm.assert_called_once_with(
            'foo', True)

    def test_alarm_find(self):
        d = ceilometer_v2.CeilometerClient(self.conn_params)
        d.alarm_find('fakeid', True)
        self.mock_conn.telemetry.find_alarm.assert_called_once_with(
            'fakeid', True)

        self.mock_conn.telemetry.find_alarm.reset_mock()

        d.alarm_find('fakeid', False)
        self.mock_conn.telemetry.find_alarm.assert_called_once_with(
            'fakeid', False)

        self.mock_conn.telemetry.find_alarm.reset_mock()

        d.alarm_find('fakeid')
        self.mock_conn.telemetry.find_alarm.assert_called_once_with(
            'fakeid', True)

    def test_alarm_get(self):
        d = ceilometer_v2.CeilometerClient(self.conn_params)
        d.alarm_get('fakeid')
        self.mock_conn.telemetry.get_alarm.assert_called_once_with('fakeid')

    def test_alarm_list(self):
        d = ceilometer_v2.CeilometerClient(self.conn_params)
        d.alarm_list()
        self.mock_conn.telemetry.alarms.assert_called_once_with()

        self.mock_conn.telemetry.alarms.reset_mock()

        d.alarm_list(name='fakename')
        self.mock_conn.telemetry.alarms.assert_called_once_with(
            name='fakename')

    def test_alarm_update(self):
        d = ceilometer_v2.CeilometerClient(self.conn_params)
        attrs = {'enabled': False}
        d.alarm_update('fakeid', **attrs)
        self.mock_conn.telemetry.update_alarm.assert_called_once_with(
            'fakeid', **attrs)

    def test_sample_create(self):
        d = ceilometer_v2.CeilometerClient(self.conn_params)
        attrs = {'foo': 'bar'}
        d.sample_create(**attrs)
        self.mock_conn.telemetry.create_sample.assert_called_once_with(
            foo='bar')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from senlin.tests.tempest.api import base
from senlin.tests.tempest.common import utils


class TestNodeCreate(base.BaseSenlinAPITest):

    def setUp(self):
        super(TestNodeCreate, self).setUp()
        self.profile_id = utils.create_a_profile(self)
        self.addCleanup(utils.delete_a_profile, self, self.profile_id)
        self.cluster_id = utils.create_a_cluster(self, self.profile_id)
        self.addCleanup(utils.delete_a_cluster, self, self.cluster_id)

    @decorators.idempotent_id('14d06753-7f0a-4ad2-84be-37fce7114a8f')
    def test_node_create_all_attrs_defined(self):
        params = {
            'node': {
                'profile_id': self.profile_id,
                'cluster_id': self.cluster_id,
                'metadata': {'k1': 'v1'},
                'role': 'member',
                'name': 'test-node'
            }
        }
        res = self.client.create_obj('nodes', params)

        # Verify resp of node create API
        self.assertEqual(202, res['status'])
        self.assertIsNotNone(res['body'])
        self.assertIn('actions', res['location'])
        node = res['body']
        self.addCleanup(utils.delete_a_node, self, node['id'])
        for key in ['cluster_id', 'created_at', 'data', 'domain', 'id',
                    'index', 'init_at', 'metadata', 'name', 'physical_id',
                    'profile_id', 'profile_name', 'project', 'role',
                    'status', 'status_reason', 'updated_at', 'user']:
            self.assertIn(key, node)
        self.assertIn('test-node', node['name'])
        self.assertEqual(self.profile_id, node['profile_id'])
        self.assertEqual(self.cluster_id, node['cluster_id'])
        self.assertEqual({'k1': 'v1'}, node['metadata'])
        self.assertEqual('member', node['role'])
        self.assertEqual('test-node', node['name'])

        # Wait node to be active before moving on
        action_id = res['location'].split('/actions/')[1]
        self.client.wait_for_status('actions', action_id, 'SUCCEEDED')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_serialization import jsonutils
from oslo_utils import encodeutils
import webob.dec

from senlin.api.common import version_request as vr


class VersionController(object):
    """WSGI controller for version in Senlin v1 API."""

    # NOTE: A version change is required when you make any change to the API.
    # This includes any semantic changes which may not affect the input or
    # output formats or even originate in the API code layer.
    _MIN_API_VERSION = "1.0"
    _MAX_API_VERSION = "1.8"

    DEFAULT_API_VERSION = _MIN_API_VERSION

    def __init__(self, conf):
        self.conf = conf

    @webob.dec.wsgify
    def __call__(self, req):
        info = self.version(req)
        body = jsonutils.dumps(info)
        response = webob.Response(request=req, content_type='application/json')
        response.body = encodeutils.safe_encode(body)

        return response

    @classmethod
    def version_info(cls):
        return {
            "id": "1.0",
            "status": "CURRENT",
            "updated": "2016-01-18T00:00:00Z",
            "media-types": [
                {
                    "base": "application/json",
                    "type": "application/vnd.openstack.clustering-v1+json"
                }
            ],
            "links": [{
                "href": "/v1",
                "rel": "self"}, {
                "rel": "help",
                "href": "https://developer.openstack.org/api-ref/clustering"
            }],
            "min_version": cls._MIN_API_VERSION,
            "max_version": cls._MAX_API_VERSION,
        }

    def version(self, req):
        return {"version": self.version_info()}

    @classmethod
    def min_api_version(cls):
        return vr.APIVersionRequest(cls._MIN_API_VERSION)

    @classmethod
    def max_api_version(cls):
        return vr.APIVersionRequest(cls._MAX_API_VERSION)

    @classmethod
    def is_supported(cls, req, min_ver=None, max_ver=None):
        """Check if API request version satisfies version restrictions.

        :param req: request object
        :param min_ver: minimal version of API needed.
        :param max_ver: maximum version of API needed.
        :returns: True if request satisfies minimal and maximum API version
                 requirements. False in other case.
        """
        min_version = min_ver or cls._MIN_API_VERSION
        max_version = max_ver or cls._MAX_API_VERSION
        return (vr.APIVersionRequest(max_version) >= req.version_request >=
                vr.APIVersionRequest(min_version))
